/*
    channel for Bt848 frame grabber driver

    Copyright (C) 1996,97 Marcus Metzler (mocm@thp.uni-koeln.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 * channel.cc
 */

#include "channel.h"
#include <string.h>
#include <qstring.h>
#include <qregexp.h>
#include <stdio.h>
#include <stdlib.h>

#define debug qDebug

int channel::version = CFVER_CUR;

channel::channel(int freq,const char *n,int c,int b,int hu,int co,int no,int in,int cntry, const char *cid)
{
  name=new char[strlen(n)+1];
  strcpy(name,n);
  frequency=freq;
  color=c;
  brightness=b;
  hue=hu;
  contrast=co;
  norm=no;
  input=in;
  country=cntry;
  channelId=new char[strlen(cid)+1];
  strcpy(channelId,cid);
}

channel::channel(const channel & c)
{
  name=new char[strlen(c.name)+1];
  strcpy(name,c.name);
  frequency=c.frequency;
  color=c.color;
  brightness=c.brightness;
  hue=c.hue;
  contrast=c.contrast;
  norm=c.norm;
  input=c.input;
  country=c.country;
  channelId=new char[strlen(c.channelId)+1];
  strcpy(channelId,c.channelId);
}


channel::~channel()
{
  delete(name);
  delete(channelId);
}

// Warning: iostream operators have bugs on some platforms so new load and
// save members are implemented below
ostream &operator<<(ostream &stream, channel &x)
{
  stream << "*" << endl;
  stream << "Chnl: " << x.name << endl;
  stream << "Frq: " << x.frequency;
  if (x.channelId)
    stream << " " << x.channelId << endl;
  else
    stream << " ??" << endl;
  stream << "CBHC: " << x.color << " " << x.brightness << " " << x.hue << " " << x.contrast << endl;
  stream << "NIS: " << x.norm << " " << x.input << endl;
  stream << "Cntry: " << x.country << endl;

  return stream;
}

istream &operator>>(istream &stream, channel &x)
{
  char cdummy[50];

  switch ( x.version ) {
  case CFVER_XTV:
    stream >> cdummy >> cdummy;
    if (x.name) delete(x.name);
    x.name=new char[sizeof(cdummy)];
    strcpy(x.name,cdummy);
    stream.get(cdummy,sizeof(cdummy)); // get full channel name
    strcat(x.name,cdummy);

    stream >> cdummy >> x.frequency;
    stream >> cdummy >> x.color >> x.brightness >> x.hue >> x.contrast;
    stream >> cdummy >> x.norm >> x.input;
    x.country=-1;
    x.channelId=new char[4];
    strcpy(x.channelId,"??");
    break;
  case CFVER_1:
    stream >> cdummy;
    stream >> cdummy;
    if (x.name) delete(x.name);
    x.name=new char[sizeof(cdummy)];
    strcpy(x.name,cdummy);
    stream.get(cdummy,sizeof(cdummy)); // get full channel name
    strcat(x.name,cdummy);

    stream >> cdummy >> x.frequency;
    stream >> cdummy >> x.color >> x.brightness >> x.hue >> x.contrast;
    stream >> cdummy >> x.norm >> x.input;
    stream >> cdummy; // old startwiththis
    x.country=-1;
    x.channelId=new char[4];
    strcpy(x.channelId,"??");
    break;
  case CFVER_2:
    stream >> cdummy >> cdummy;  // * \n Channel:
    if (x.name) delete(x.name);
    x.name=new char[255];
    strcpy(x.name,cdummy);
    stream.get(cdummy,sizeof(cdummy)); // get full channel name
    strcat(x.name,cdummy);

    stream >> cdummy >> x.frequency;
    stream >> cdummy >> x.color >> x.brightness >> x.hue >> x.contrast;
    stream >> cdummy >> x.norm >> x.input;
    stream >> cdummy >> x.country;
    x.country=-1;
    x.channelId=new char[4];
    strcpy(x.channelId,"??");
    break;
  case CFVER_3:
    stream >> cdummy >> cdummy;  // * \n Channel:

    if (x.name) delete(x.name);
    x.name=new char[255];
    strcpy(x.name,cdummy);
    stream.get(cdummy,sizeof(cdummy)); // get full channel name
    strcat(x.name,cdummy);

    stream >> cdummy >> x.frequency >> cdummy;

    if (x.channelId) delete(x.channelId);
    x.channelId=new char[255];
    strcpy(x.channelId,cdummy);
    stream.get(cdummy,sizeof(cdummy)); // get full channelId
    strcat(x.channelId,cdummy);

    stream >> cdummy >> x.color >> x.brightness >> x.hue >> x.contrast;
    stream >> cdummy >> x.norm >> x.input;
    stream >> cdummy >> x.country;
    break;
  default:
    cerr << "Warning: Unsupported channel file format " << x.version << endl;
  }

  return stream;
}






bool channel::save( QTextStream *stream, int ver )
{
/* Print some qDebugging information before we save it to the stream:
	qDebug( "=====================================" );
	qDebug( "SAVING CHANNEL!" );
	qDebug( "Name: %s", name );
	qDebug( "Freq: %d", frequency );
	qDebug( "ChannelId: %s", channelId );
	qDebug( "Color: %d", color );
	qDebug( "Brightness: %d", brightness );
	qDebug( "Hue: %d", hue );
	qDebug( "Contrast: %d", contrast );
	qDebug( "Norm: %d", norm );
	qDebug( "Input: %d", input );
	qDebug( "Country: %d", country );
*/

	// Saving version 3 (0.7.5) -----------------------------------------

	if ( ver == CFVER_3 ) {
		*stream << "*" << endl;
	   *stream << TAG_CHANNEL << name << endl;
	   *stream << TAG_FREQ << frequency;
	   if ( channelId ) {
	     *stream << " " << channelId << endl;
	   }
	   else {
	     *stream << " ??" << endl;
	   }
	   *stream << TAG_CBHC << color << " " << brightness << " " << hue << " " << contrast << endl;
	   *stream << TAG_NIS << norm << " " << input << endl;
	   *stream << TAG_CNTRY << country << endl;
  } // end if, ver = 3

  // Saving version 4 (0.7.6, Stefan's Patch) --------------------------

	if ( ver == CFVER_4 ) {
  		*stream << "*" << endl;
	  	*stream << TAG_CHANNEL << name << endl;
		*stream << TAG_FREQ << frequency << endl;
		*stream << TAG_CHID << channelId << endl;
		*stream << TAG_COLOR << color << endl;
		*stream << TAG_BRIGHT << brightness << endl;
		*stream << TAG_HUE << hue << endl;
		*stream << TAG_CONTR << contrast << endl;
		*stream << TAG_NORM << norm << endl;
		*stream << TAG_INPUT << input << endl;
		*stream << TAG_CNTRY << country << endl;
	}	// end if, ver = 4

return true;
}

// LOAD: member function that loads a channel from a QTextStream
// Caution! Only parsers for versions #3 and #4 (new) have been
// tested as of now, please be careful with file versions xtv, 1, and 2.

// FIXME: this needs a LOT of work.  lines starting with "[ ]*#" or blank lines
//        need to be entirely ignored
//        Get rid of sscanf()!

bool channel::load( QTextStream *stream, int vers )
{
	QString str;		// Temporary QString needed by all parsers
	char *tmpname;		// Required by parser version #2, #1, and XTV
	int result = 0;			// For sscanf
	bool keepreading = false;	// Required by parser version #3
	bool status = true;	// Required by parser version #4 and others
	int entries;			// Required by parser version #4
        QRegExp anycomment("^[ ]*#.*$");

	// Check if there's an asterisk in the line, here begins a new channel:
	// This is true for all channel definition files, I hope :-)
	// Otherwise just delete the if-line(s) below and just ignore the line.
	// I check for a "*" to make sure we're actually reading a channel file.

  // This helps us ignore lines we don't need to read so the parse works better
#define read_a_string(x, y)    x = "";                                       \
        while (x.isEmpty() || x.contains(anycomment))                        \
          if (y->eof()) return false;                                        \
             else x = y->readLine();                                          


        read_a_string(str, stream);

        // Skip * \n -> New channel!
	if (str.startsWith("*")) {
		status = true;
	} else {
		qDebug( "Error in kwintv programs file, asterisk is missing!" );
		return( false );
	}
	// Enter the parser depending on the version information we have:

	switch ( vers ) {

	case CFVER_CUR:	// current version (4) ----------------------------

		// Parse Stefan's new channel file!

		entries = 10;	// Entries to parse in new file format: 10
			// This way the file format can store its channel
			// data in any order it likes, no sequence is required
			// If there's an error while parsing, status is false.
		while ( entries && status )
		{
                        read_a_string(str, stream);

			// Parse channel name:
			if ( str.find( TAG_CHANNEL, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_CHANNEL ) );
				if ( name ) delete ( name );
				name = new char[ MAX_STR ];
				strcpy( name, str.latin1() );			// Store name
				status = true; entries--;
			}

			// Parse Frequency
			if ( str.find( TAG_FREQ, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_FREQ ) );
				frequency = str.toInt(&status);
				entries--;
			}

			// Parse ChannelID
			if ( str.find( TAG_CHID, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_CHID ) );
				if (channelId) delete ( channelId );
				channelId = new char[ MAX_STR ];
				strcpy( channelId, str.latin1() );		// Store ChID
				status = true; entries--;
			}

			// Parse Color
			if ( str.find( TAG_COLOR, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_COLOR ) );
				color = str.toInt(&status);
				entries--;
			}

			// Parse Brightness
			if ( str.find( TAG_BRIGHT, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_BRIGHT ) );
				brightness = str.toInt(&status);
				entries--;
			}

			// Parse Hue
			if ( str.find( TAG_HUE, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_HUE ) );
				hue = str.toInt(&status);
				entries--;
			}

			// Parse Contrast
			if ( str.find( TAG_CONTR, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_CONTR ) );
				contrast = str.toInt(&status);
				entries--;
			}

			// Parse Norm
			if ( str.find( TAG_NORM, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_NORM ) );
				norm = str.toInt(&status);
				entries--;
			}

			// Parse Input source
			if ( str.find( TAG_INPUT, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_INPUT ) );
				input = str.toInt(&status);
				entries--;
			}

			// Parse Country
			if ( str.find( TAG_CNTRY, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_CNTRY ) );
				country = str.toInt(&status);
				entries--;
			}
		}		// end while
		break;	// end case (parser version 4)

	case CFVER_3:		// PARSER FOR VERSION 3 ----------------------

		if ( status ) keepreading = true;
		while ( keepreading )
		{
                        read_a_string(str, stream);

			if ( str.find( TAG_CHANNEL, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_CHANNEL ) );
				if ( name ) delete ( name );
				name = new char[ MAX_STR ];
				strcpy( name, str.latin1() );	// Store str in name
				keepreading = true;
			}

			if ( str.find( TAG_FREQ, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_FREQ ) );
				if (str.find(' ') == -1) return ( false );
                                frequency = str.left(str.find(' ')).toInt();
                                str.remove(0,str.find(' ')+1);
				if (channelId) delete ( channelId );
				channelId = new char[ MAX_STR ];
				strcpy( channelId, str.latin1() );
				keepreading = true;
			}

			if ( str.find( TAG_CBHC, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_CBHC ) );
				result = sscanf( str.latin1(), "%d %d %d %d", &color, &brightness, &hue, &contrast );
				if ( result != 4 ) return ( false );
				keepreading = true;
			}
			if ( str.find( TAG_NIS, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_NIS ) );
				result = sscanf( str.latin1(), "%d %d", &norm, &input );
				if ( result != 2 ) return ( false );
				keepreading = true;
			}
			if ( str.find( TAG_CNTRY, 0, false ) != -1 ) {
				str.remove( 0, strlen( TAG_CNTRY ) );
				country = str.toInt();
				keepreading = false;		// End of channel definition!
			}

		} 		// end while
		break; 	// end case (parser version 3)

	case CFVER_2:		// PARSER FOR VERSION 2 -------------------
	// The following has not been tested yet!!!

		str = stream->readLine();	// Read another line of channel defs
		// Parse channel name:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %s", tmpname );
		if ( result != 1 ) return ( false );
		if ( name ) delete ( name );
		name = new char[ MAX_STR ];
		strcpy( name, tmpname );

		str = stream->readLine();	// Read another line of channel defs
		// Parse frequency:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d", &frequency );
		if ( result != 1 ) return ( false );

		str = stream->readLine();	// Read another line of channel defs
		// Parse color, brightness, hue, contrast:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d %d %d %d", &color, &brightness, &hue, &contrast );
		if ( result != 4 ) return ( false );

		str = stream->readLine();	// Read another line of channel defs
		// Parse norm, input:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d %d", &norm, &input );
		if ( result != 2 ) return ( false );

		str = stream->readLine();	// Read another line of channel defs
		// Parse country:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d", &country );
		if ( result != 1 ) return ( false );

		// For backwards compatibility with 0.7.5:
		country = -1;
		channelId = new char[4];
		strcpy( channelId, "??" );
		break;	// end version #2

	case CFVER_1:		// PARSER FOR VERSION 1 ---------------------
	// The following has not been tested yet!!!

		str = stream->readLine();	// Read another line of channel defs
		// Parse channel name:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %s", tmpname );
		if ( result != 1 ) return ( false );
		if ( name ) delete ( name );
		name = new char[ MAX_STR ];
		strcpy( name, tmpname );

		str = stream->readLine();	// Read another line of channel defs
		// Parse frequency:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d", &frequency );
		if ( result != 1 ) return ( false );

		str = stream->readLine();	// Read another line of channel defs
		// Parse color, brightness, hue, contrast:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d %d %d %d", &color, &brightness, &hue, &contrast );
		if ( result != 4 ) return ( false );

		str = stream->readLine();	// Read another line of channel defs
		// Parse norm, input:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d %d", &norm, &input );
		if ( result != 2 ) return ( false );

		str = stream->readLine();	// Read another line of channel defs
		// Just ignore this last line ("old startwiththis")

		// For backwards compatibility with 0.7.5:
		country = -1;
		channelId = new char[4];
		strcpy( channelId, "??" );
		break;	// end version #1

	case CFVER_XTV:		// PARSER FOR XTV FILES ----------------------
	// The following has not been tested yet!!!

		str = stream->readLine();	// Read another line of channel defs
		// Parse channel name:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %s", tmpname );
		if ( result != 1 ) return ( false );
		if ( name ) delete ( name );
		name = new char[ MAX_STR ];
		strcpy( name, tmpname );

		str = stream->readLine();	// Read another line of channel defs
		// Parse frequency:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d", &frequency );
		if ( result != 1 ) return ( false );

		str = stream->readLine();	// Read another line of channel defs
		// Parse color, brightness, hue, contrast:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d %d %d %d", &color, &brightness, &hue, &contrast );
		if ( result != 4 ) return ( false );

		str = stream->readLine();	// Read another line of channel defs
		// Parse norm, input:
		// Scan entry and always ignore the first tag
		result = sscanf( str.latin1(), "%*s %d %d", &norm, &input );
		if ( result != 2 ) return ( false );

		// For backwards compatibility with 0.7.5:
		country = -1;
		channelId = new char[4];
		strcpy( channelId, "??" );
		break;	// end version xtv

	default:
		qDebug( "Error! Channel file version %d not supported by parser!", vers );
		return( false );
		break;
	}			// end switch

	/* QDebug information about the channel we read:
	qDebug( "===============================================" );
	qDebug( "CHANNEL READ (version %d):", vers );
	qDebug( "Name = %s", name );
	qDebug( "Frequency = %d", frequency );
	qDebug( "ChannelId = %s", channelId );
	qDebug( "Color = %d", color );
	qDebug( "Brightness = %d", brightness );
	qDebug( "Hue = %d", hue );
	qDebug( "Contrast = %d", contrast );
	qDebug( "Norm = %d", norm );
	qDebug( "Input = %d", input );
	qDebug( "Country = %d", country );
	*/

	return( status );	// Status is only touched by parser #4 and "*" check.
}

