/* 
    Copyright (C) 2001 Stefan Hellwig <stefan@stefanhellwig.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
#include "channellist.h"

// =======================================================================
// Default constructor. Right now it doesn't do much since
// the channel list itself is created by default anyway...
// =======================================================================

ChannelList::ChannelList( ) {
	chConfig = NULL;
}	

// =======================================================================
// Constructor accepting a file name for the KConfig file
// =======================================================================

ChannelList::ChannelList( const QString& fileName, bool legacyMode ) {
	if ( !legacyMode ) {
		chConfig = new KConfig( fileName, false, false, "data" );
	} else {
		chConfig = NULL;
	}
}

// =======================================================================
// Creates the KConfig object later on (for legacy mode support)
// using the given filename. Obsolete!
// =======================================================================

void ChannelList::createConfig( const QString& fileName ) {
	chConfig = new KConfig( fileName, false, false, "data" );
}

// =======================================================================
// "Destroys" a current KConfig object
// =======================================================================

void ChannelList::closeConfig() {
	chConfig = NULL;
}

// =======================================================================
// Load the channels and fill the channel list
// =======================================================================

bool ChannelList::load() {
	if ( chConfig == NULL ) return false;
	QString groupName;
	channel *newChannel;
	int counter = 1;
	for ( ;; ) {
	// Add the counter index to the channel's group name
    	groupName.sprintf( CHANNEL_GROUP, counter );
		if ( chConfig->hasGroup( groupName ) ) {
			chConfig->setGroup( groupName );
			newChannel = readChannel( groupName );
			if ( newChannel == NULL ) {
				return( false );
			} else {
			// Append the newly created channel to the list
			append( newChannel );
			}
		} else {
			return( false );
		}
		counter++;
	}
	return( true );
}

// =======================================================================
// Save all channels in the list to the default file
// =======================================================================

bool ChannelList::save() {
	if ( isEmpty() ) return false;
	if ( chConfig == NULL ) {
		// This should only happen in legacy mode or during "Save Channels As"!
		createConfig( QString( "default" ) );
	}
	QString groupName;
	channel *ch = first();
	int counter = 1;
	groupName.sprintf( CHANNEL_GROUP, counter );
	writeChannel( groupName, ch );
	while( (ch = next()) != NULL ) {
		counter++;
		groupName.sprintf( CHANNEL_GROUP, counter );
		writeChannel( groupName, ch );
	}
	return( true );
}

// =======================================================================
// Save all channels in the list to a file with the given name
// =======================================================================

bool ChannelList::saveAs( const QString& fileName ) {
	if ( isEmpty() ) return false;
	if ( chConfig == NULL ) {
		// This should only happen in legacy mode or during "Save Channels As"!
		createConfig( fileName );
	}
	QString groupName;
	channel *ch = first();
	int counter = 1;
	groupName.sprintf( CHANNEL_GROUP, counter );
	writeChannel( groupName, ch );
	while( (ch = next()) != NULL ) {
		counter++;
		groupName.sprintf( CHANNEL_GROUP, counter );
		writeChannel( groupName, ch );
	}
	return( true );
}

// =======================================================================
// Create and read a channel from the KConfig channel file
// =======================================================================

channel *ChannelList::readChannel( QString &groupName ) {
	QString channelName;
	int freq = 0;
	QString chID = 0;
	int color = 0;
	int brightness = 0;
	int hue = 0;
	int contrast = 0;
	int norm = 0;
	int input = 0;
	int country = 0;
	channelName = chConfig->readEntry( CHANNEL_NAME, "unknown" );
	freq = chConfig->readNumEntry( CHANNEL_FREQ, 0 );
	chID = chConfig->readEntry( CHANNEL_ID, "??" );
	color = chConfig->readNumEntry( CHANNEL_COLOR, 0 );
	brightness = chConfig->readNumEntry( CHANNEL_BRIGHT, 0 );
	hue = chConfig->readNumEntry( CHANNEL_HUE, 0 );
	contrast = chConfig->readNumEntry( CHANNEL_CONTR, 0 );
	norm = chConfig->readNumEntry( CHANNEL_NORM, 0 );
	input = chConfig->readNumEntry( CHANNEL_INPUT, 0 );
	country = chConfig->readNumEntry( CHANNEL_COUNTRY, 0 );
	channel *ch = new channel( freq, (const char *)channelName, color, brightness, hue,
					           contrast, norm, input, country, (const char *)chID );
	return ch;
}

// =======================================================================
// Save a channel to the KConfig channel file
// =======================================================================

void ChannelList::writeChannel( QString &groupName, channel *ch ) {
	if ( (chConfig == NULL) || (ch == NULL) ) return;
	chConfig->setGroup( groupName );
	chConfig->writeEntry( CHANNEL_NAME, ch->name );
	chConfig->writeEntry( CHANNEL_ID, ch->channelId );
	chConfig->writeEntry( CHANNEL_FREQ, ch->frequency );
	chConfig->writeEntry( CHANNEL_COLOR, ch->color );
	chConfig->writeEntry( CHANNEL_BRIGHT, ch->brightness );
	chConfig->writeEntry( CHANNEL_HUE, ch->hue );
	chConfig->writeEntry( CHANNEL_CONTR, ch->contrast );
	chConfig->writeEntry( CHANNEL_NORM, ch->norm );
	chConfig->writeEntry( CHANNEL_INPUT, ch->input );
	chConfig->writeEntry( CHANNEL_COUNTRY, ch->country );
	chConfig->sync();
}	
