/*
    Copyright (C) 2001 Stefan Hellwig <stefan@stefanhellwig.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef __CHANNELLIST_H__
#define __CHANNELLIST_H__

#include <channel.h>
#include <qlist.h>
#include <qstring.h>
#include <kconfig.h>
#include <string.h>

// The CHANNEL_GROUP definition actually defines how
// the channel group name in the KConfig-based file is
// set up.

#define CHANNEL_GROUP   "channel %d"

// Define the different tags/keys within a channel group

#define CHANNEL_NAME    "ChannelName"
#define CHANNEL_FREQ    "Frequency"
#define CHANNEL_ID      "ChannelId"
#define CHANNEL_COLOR   "Color"
#define CHANNEL_BRIGHT  "Brightness"
#define CHANNEL_HUE     "Hue"
#define CHANNEL_CONTR   "Contrast"
#define CHANNEL_NORM    "Norm"
#define CHANNEL_INPUT   "Input"
#define CHANNEL_COUNTRY "Country"

class ChannelList : public QList<channel>
{
public:
	ChannelList( );
	ChannelList( const QString& fileName, bool legacyMode );
	void createConfig( const QString& fileName );
	void closeConfig();

	bool load();
	bool save();
	bool saveAs( const QString& fileName );
	
private:
	KConfig *chConfig;
	QString *fileName;

	channel *readChannel( QString& groupName );
	void writeChannel( QString& groupName, channel* ch );
};

#endif
