/* 
    KDE Xv interface

    Copyright (C) 2001 George Staikos (staikos@kde.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifndef __KXV_H
#define __KXV_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <X11/X.h>
#include <X11/Xlib.h>
#include <qstring.h>
#include <qlist.h>

class QWidget;


class KXvPrivate;
class KXvDevice;
class KXvDevicePrivate;

typedef QList<KXvDevice> KXvDeviceList;


class KXv {
public:
  ~KXv();

  /*
   *   To get access to the Xv extension, you call this method.  It will return
   *   a KXv* object on success, or NULL if it can't connect.
   *
   *   d is typically the Window ID
   */
  static KXv *connect(Drawable d);

  /*
   *   True if we can connect to the Xv extension.
   */
  static bool haveXv();

  /*
   *   Return the list of Xv devices
   */
  KXvDeviceList& devices();

protected:
  KXv();
  bool init(Drawable d);

  /***   XV info   ***/
  unsigned int xv_version, xv_release, xv_request, xv_event, xv_error;
  unsigned int xv_adaptors;
  void *xv_adaptor_info;


  KXvDeviceList _devs;


private:
  KXvPrivate *d;
};



class KXvDeviceAttribute {
public:
  QString name;
  int min;
  int max;
  int flags;

  Atom atom();
};

typedef QList<KXvDeviceAttribute> KXvDeviceAttributes;



class KXvDevice {
friend class KXv;
public:

  KXvDevice();
  ~KXvDevice();

  /*
   *   return the list of known attributes
   */
  const KXvDeviceAttributes& attributes();

  /*
   *   return the range for a given attribute
   */
  bool getAttributeRange(QString attribute, int *min, int *max);

  /*
   *   get the current value of a given attribute
   */
  bool getAttribute(QString attribute, int *val);

  /*
   *   set the current value of a given attribute
   */
  bool setAttribute(QString attribute, int val);

  /*
   *   Start a video stream in widget w, width dw, height dh
   */
  bool startVideo(QWidget *w, int dw, int dh);
  bool startVideo(Window w, int dw, int dh);

  /*
   *   Stop video stream
   */
  bool stopVideo();

  /*
   *   True if this is a video source
   */
  bool isVideoSource();

  /*
   *   Name of the device
   */
  const QString& name() const;


protected:

  bool init();

  KXvDeviceAttributes _attrs;

  int xv_type, xv_adaptor;
  QString xv_name;
  int xv_port;
  unsigned int xv_encodings;
  int xv_encoding;
  void *xv_encoding_info;
  int xv_encoding_attributes;
  void *xv_attr;
  GC xv_gc;

  int xv_formats;
  void *xv_formatvalues;

  bool videoStarted;
  Window videoWindow;

private:
  KXvDevicePrivate *d;
};


#endif

