//-> Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)
/*
    xWinTV using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
//-> includes
#include <stdio.h>
#include <stdlib.h>

#include "wintv.h"
#include "channellist.h"

#include "xrwevent.h"
#include "wintvmain.h"
#if defined ( HAVE_VTX ) || defined ( HAVE_VBI )
#include <winvtxmain.h>
#endif
#include "wintvchannel.h"
#include "wintvstream.h"

#include <kapp.h>

#include <klocale.h>
#include <khelpmenu.h>
#include <kglobal.h>
#include <kstddirs.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <dcopclient.h>
#include <kdebug.h>

// X11 stuff
#include <X11/Xlib.h>


TVApplication::TVApplication( int & argc, char ** argv ) : KApplication(argc, argv)
{
  rc = new rcfile();
  rc->load();

  winTVMain = new winTVMainWidget( rc );
  winTVChannel = new winTVChannelWidget( rc );
  winTVStream = new winTVStreamWidget( rc );

  for (int i = 0; i < mixer::getNumMixer(); i++) {
    winTVMixer[i] = new winMixer(rc);
    connect( winTVMixer[i], SIGNAL(popDown(int)), winTVMain, SLOT(mixerDown(int)) );
  }

  setFilterObject(winTVMain->winTVScreenPtr());
  setMainWidget(winTVMain);

  // connect TVApplication
  connect(this,SIGNAL(shutDown()),this,SLOT(shutDownSLOT()));
  connect(this,SIGNAL(saveYourself()),this,SLOT(saveYourselfSLOT()));

  // connect winTVMain and winTVChannel
  connect(winTVMain,SIGNAL(channelChangedSIGNAL()),
		   winTVChannel,SLOT(activeChannelChangedSLOT()));
  connect(winTVMain,SIGNAL(newChannelListSIGNAL(ChannelList *)),
		   winTVChannel,SLOT(newChannelListSLOT(ChannelList *)));
  connect(winTVMain,SIGNAL(toggleChannelBoxSIGNAL( QPoint * )),
		   winTVChannel,SLOT(toggleVisibilitySLOT( QPoint * )));
  connect(winTVChannel, SIGNAL(popDown()), winTVMain, SLOT(channelBoxDownSLOT()) );

  if (rc->mix_master >= 0 && rc->mix_master < mixer::getNumMixer() && winTVMixer[rc->mix_master]) {
     // connect winTVMixer and winTVMain
     connect(winTVMixer[rc->mix_master],SIGNAL(volChanged(int, int)),
	  winTVMain,SLOT(volumeSLOT(int, int)));
     connect(winTVMixer[rc->mix_master],SIGNAL(volMuteChanged(bool)),
	  winTVMain,SLOT(volMuteChangedExtSLOT(bool)));
     // connect winTVMain and winTVMixer
     connect(winTVMain,SIGNAL(volumeChangedSIGNAL(int,int)),
	  winTVMixer[rc->mix_master],SLOT(extVolSet(int,int)));
     connect(winTVMain,SIGNAL(volMuteChangedSIGNAL(bool)),
	  winTVMixer[rc->mix_master],SLOT(extVolMute(bool)));
     connect(winTVMain,SIGNAL(toggleMixerBoxSIGNAL(int)),
  	  this,SLOT(showMixerSLOT(int)));
  }

  if (rc->mix_master >= 0 && rc->mix_master < mixer::getNumMixer() && winTVMixer[rc->mix_master]) {
    // connect winTVScreen and winTVMixer
    connect(winTVMain->winTVScreenPtr(),SIGNAL(requestMixerMute(bool)),
	  winTVMixer[rc->mix_master],SLOT(extMuteRequest( bool )));
  }

  // edit, cut, copy and paste channel
  connect(winTVChannel,SIGNAL(channelSelectedSIGNAL()),
	  winTVMain,SLOT(channelChangedSLOT()));
  connect(winTVChannel,SIGNAL(addChannelSIGNAL()),
	  winTVMain,SLOT(addChannelSLOT()));
  connect(winTVChannel,SIGNAL(editChannelSIGNAL()),
	  winTVMain,SLOT(editChannelSLOT()));
  connect(winTVChannel,SIGNAL(deleteChannelSIGNAL()),
	  winTVMain,SLOT(deleteChannelSLOT()));
  connect(winTVChannel,SIGNAL(copyChannelSIGNAL()),
	  winTVMain,SLOT(copyChannelSLOT()));
  connect(winTVChannel,SIGNAL(pasteChannelSIGNAL()),
	  winTVMain,SLOT(pasteChannelSLOT()));

  // connect winTVMain and winTVVClip
  connect(winTVMain,SIGNAL(toggleVClipSIGNAL()),
		   winTVStream,SLOT(toggleVClipSLOT()));

//  winTVChannel->setPopup(winTVMain->getChannelMenu());
  winTVChannel->setPopup( (KPopupMenu*)winTVMain->guiFactory()->container( "prog", winTVMain ) );

  winTVMain->show();

  KConfig *config = kapp->config();
  for (int i = 0; i < mixer::getNumMixer(); i++ ) {
    config->setGroup("CONFIG__mixer-"+QString().setNum(i));
    //winTVMixer[rc->mix_master]->setV4lifP(winTVMain->winTVScreenPtr()->v4lxifP());
    if ( config->readNumEntry("was_visible",0) && winTVMixer[i]->mixerEnabled() )
      winTVMixer[i]->show();
  }

  if ( rc->ol_ch_wasvis ) winTVChannel->show();

  winTVMain->emitSetupSignals();

#ifdef HAVE_VTX
  winVTXMain = new winVTXMainWidget( pcl );
  winVTXMain->show();
  winVTXMain->postInit();
#endif

  shutdownDone = false;
}

void TVApplication::showMixerSLOT(int id)
{
  if (id < 0 || id >= mixer::getNumMixer()) 
     return;
  winTVMixer[id]->showMixer();
}

void TVApplication::destroyTV()
{
  if ( shutdownDone ) return;
  shutdownDone = true;

  setFilterObject(NULL);

  delete winTVStream;
  rc->ol_ch_wasvis = winTVChannel->isVisible();
  delete winTVChannel;
  delete winTVMain;

  KConfig *config= kapp->config();
  for ( int i = 0; i < mixer::getNumMixer(); i++ ) {
    config->setGroup("CONFIG__mixer-"+QString().setNum(i));
    config->writeEntry("was_visible",(int)winTVMixer[i]->isVisible());
    delete winTVMixer[i];
  }

#ifdef HAVE_VTX
  delete winVTXMain;
#endif

  delete rc;

  rc = NULL;
}

TVApplication::~TVApplication()
{
  destroyTV();
}

//-> bool TVApplication::x11EventFilter( XEvent *event)
bool TVApplication::x11EventFilter( XEvent *event )
{
  Window checkw1,checkw2,wwin;

  wwin=DefaultRootWindow(qt_xdisplay());
  checkw1=event->xmap.event;
  checkw2=event->xany.window;

  if (checkw1 == wwin) {

    X11RWEvent *xrwe = new X11RWEvent(XRWEvent,
				      event->type,
				      event);
    postEvent(winTVscreenObject,xrwe);

    return TRUE;
  } else if (checkw2 == wwin) {
    return TRUE;
  }

 return FALSE;
}

void TVApplication::shutDownSLOT()
{
  destroyTV();
}

void TVApplication::saveYourselfSLOT()
{

  if ( rc != NULL )
    rc->save();
}

bool TVApplication::queryExit()
{
  destroyTV();
  return true;
}

void TVApplication::sessionManagerSLOT()
{
}

void myMessageOutput( QtMsgType type, const char* msg )
{
  switch ( type ) {
  case QtDebugMsg:
    kdDebug() << "Fatal: " << msg << endl;
    break;
  case QtWarningMsg:
    kdWarning() << "Fatal: " << msg << endl;
    break;
  case QtFatalMsg:
    kdError() << "Fatal: " << msg << endl;
    //abort();                        // dump core on purpose
    exit(1);
  }
}


int main( int argc, char **argv )
{
  qInstallMsgHandler( myMessageOutput );

  KAboutData aboutData("kwintv","KWinTV",
		       KWINTV_VERSION,
		       "KWinTV is a KDE application for TV cards\n"
		       "supported by a Video4Linux compatible driver.",
		       KAboutData::License_GPL,
		       "Copyright 1998, 1999, 2000 by Moritz Wenk\n"
                       "Copyright 2001, George Staikos",
		       0,
		       "http://www.stefanhellwig.de/kwintv/index.html");

  aboutData.addAuthor("Moritz Wenk",0,"wenk@swyx.com","http://kwintv.sourceforge.net");
  aboutData.addAuthor("George Staikos",0,"staikos@kde.org", "http://www.staikos.on.ca/~staikos/kwintv/");
  aboutData.addAuthor("Stefan Hellwig",0,"stefan@stefanhellwig.de", "http://www.stefanhellwig.de/kwintv/index.html");

  KCmdLineArgs::init( argc, argv, &aboutData );

  static KCmdLineOptions options[] = {
	{ "bpp <pits per pixel>", I18N_NOOP("color depth of the display is <pits per pixel>"), 0 },
	{ "b", 0, 0 },
	{ "palette <id>", I18N_NOOP("force used palette for display to <id>"), 0 },
	{ "p", 0, 0 },
	{ "shift <int>", I18N_NOOP("shift frame buffer (TV image) by <int> bytes"), 0},
	{ "s", 0, 0 },
	{ "refresh <mode>", I18N_NOOP("set refresh mode to <mode>"), 0 },
	{ "r", 0, 0 },
	{ "device <dev>", I18N_NOOP("use device <dev> as video4linux device"), "/dev/video" },
	{ "d", 0, 0 },
	{ "noxfree", I18N_NOOP("disable XFree extentions (VidMode, DGA, DV)"), 0 },
	{ "x", 0 ,0 },
	{ "usexv", I18N_NOOP("Use Xv extension if available"), 0 },
	{ "u", 0 ,0 }
  };
  KCmdLineArgs::addCmdLineOptions(options);

  KApplication::setColorSpec( KApplication::ManyColor );

  TVApplication *winTVApplication = new TVApplication( argc, argv );

  winTVApplication->dcopClient()->attach();
  winTVApplication->dcopClient()->registerAs("kwintv", false);
  winTVApplication->dcopClient()->setDefaultObject("KWinTVIface");
  // main loop
  winTVApplication->exec();

  // the end
  delete winTVApplication;

  return 0;
}


#include "wintv.moc"






