/*
    kwintv, Video4Linux based TV application

    Copyright (C) 1998,1999 Moritz Wenk (wenk@mathematik.uni-kl.de)
    Copyright (C) 2001 George Staikos (staikos@kde.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// Hey, better do not change the order of the includes !!!
// REMEMBER: KDE & Qt-stuff BEFORE X11-stuff !!!

/*
README: this is very confusing.  channelUpSLOT() actually means go down one channel
        channelDownSLOT() goes up a channel
        in addition, switchToChannelSLOT(int) goes to channel ((int)-1)
*/
#include <wintv.h>

#include <unistd.h>
#include <assert.h>

#include <qtooltip.h>
#include <qfiledlg.h>
#include <qkeycode.h>
#include <kmessagebox.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qlayout.h>

#include <wintvmain.h>

#include <kiconloader.h>
#include <klocale.h>
#include <kwin.h>
#include <kmenubar.h>
#include <khelpmenu.h>
#include <kglobal.h>
#include <kstddirs.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <kdebug.h>
#include <kfiledialog.h>
#include <dcopclient.h>
#include <kdatastream.h>
#include <kaction.h>
#include <kstdaction.h>
#include <ktoolbar.h>
#include <ktoolbarbutton.h>
#include <kapp.h>

#include <wintvscreen_xv.h> 
#include <countries.h>
#include "channellist.h"

#include <X11/Xlib.h>
#include <X11/Xutil.h>

//#undef HAVE_LIBXXF86VM
#if defined(HAVE_LIBXXF86VM)
#include <X11/extensions/xf86vmode.h>
#include <X11/extensions/xf86vmstr.h>
XF86VidModeModeInfo **vm_modelines;
int vm_count;
#endif

#ifdef HAVE_LIBXDPMS
/* this library doesn't even have its own prototypes ... */
extern "C" {
#include <X11/extensions/dpms.h>
}
extern BOOL dpms_on;
#endif

#include "xhelpers.h"

#ifdef X11DEBUG
#include "xeventtype.h"
#endif

#define BID_UP      0
#define BID_DOWN    1
#define BID_CAP     2
#define BID_SS      3
#define BID_FAR     4
#define BID_FSS     5
#define BID_VUP     6
#define BID_MUTE    7
#define BID_VDOWN   8
#define BID_SC      9
#define BID_EXIT   10
#define BID_LCD    11
#define BID_VSL    12

#define SB_PROG_TEXT  1
#define SB_PROG       2
#define SB_SNAP_TEXT  3
#define SB_SNAP       4
#define SB_VOL_TXT    5
#define SB_VOL        6
#define SB_SIZE_TXT   7
#define SB_SIZE       8

#define ID_HIDEMENU   0
#define ID_HIDETOOL   1
#define ID_HIDESTATUS 2
#define ID_HIDEALL    3
#define ID_ALWAYSTOP  4

#define ID_TVCAPTURE    0
#define ID_TVFULLSCREEN 1
#define ID_TVSIZE       2

#define AU_MUTE       6
#define AU_MIXER      7
#define AU_MIXER_ID   256

#define RGBCOLOR (unsigned int)  //QRgb

winTVMainWidget::winTVMainWidget( rcfile *_rc, QWidget* parent, const char* name )
  :  KMainWindow( parent, name, Qt::WStyle_Customize|Qt::WStyle_NormalBorder|Qt::WStyle_Minimize|Qt::WStyle_Title|Qt::WStyle_SysMenu), DCOPObject("KWinTVIface"), rc(_rc)
{
  // some default settings
  isfullscreen = false;
  isfullsize = false;
  capture = rc->capture;
  fixed = false;
  lastCutCopy = 0;
  copyChannel = false;
  externalMute = false;
  chfile_was_changed = false;
  lcd = 0;
  winTVScreen = NULL;
  scanwizard = NULL;
  chup_disabled = false;
  chdown_disabled = false;
  m_bPredefSizeSet = false;

  setupActions();

  setCaption(i18n("KWinTV ")+QString(VERSION));

dock_widget = NULL;
  if ( rc->ol_allowdocking ) {
    dock_widget = new KDE2DockWidget(this, "KDE2DockWidget");
    connect(dock_widget, SIGNAL(fullScreenRequest()), this, SLOT(fullScreenSLOT()));
    connect(dock_widget, SIGNAL(channelsRequest()), this, SLOT(toggleChannelBoxSLOT()));
    dock_widget->dock();
  }

  // set icons
  setIconText(i18n("KWinTV"));
  QWidget::setIcon( BarIcon("kwintv_48x48") );

  QToolTip::setEnabled(rc->ol_tooltips);

  QStrList *resolution_list = NULL;
#ifdef HAVE_LIBXXF86VM
XLockDisplay(qt_xdisplay());
  Display *dpy = qt_xdisplay();
  int  foo, bar, ma, mi;
  char dummy[255];
  resolution_list = new QStrList();

  if (XF86VidModeQueryExtension(dpy,&foo,&bar)) {
    XF86VidModeQueryVersion(dpy,&ma,&mi);
    if (ma == 0 && mi < 8) {
      kdWarning() << "main: VideoModeExtension Version is " << ma << "." << mi << endl;
      kdWarning() << "main: Version 0.8 or newer required -> disabled use of VideoModeExtension" << endl;
      have_vm = false;
    } else {
      have_vm = true;
// FIXME: this needs to be done for each monitor and stored separately
      XF86VidModeGetAllModeLines(dpy,XDefaultScreen(dpy),&vm_count,&vm_modelines);
      kdDebug() << "main: X-Server supports VidMode extension (version " << ma << ", " << mi << ")" << endl;;
      for (int i = 0; i < vm_count; i++) {
	sprintf(dummy,"%d x %d",vm_modelines[i]->hdisplay,vm_modelines[i]->vdisplay);
	resolution_list->append(dummy);
      }
    }
  }
XUnlockDisplay(qt_xdisplay());
#endif

  userKeys = new KAccel( this );

  userKeys->insertItem(i18n("previous channel"),"previous channel",Key_BackSpace);
  userKeys->connectItem("previous channel",this,SLOT(channelPreviousSLOT()));

  userKeys->insertItem(i18n("program up"),"program up",Key_Down);
  userKeys->connectItem("program up",this,SLOT(channelDownSLOT()));

  userKeys->insertItem(i18n("program down"),"program down",Key_Up);
  userKeys->connectItem("program down",this,SLOT(channelUpSLOT()));

  userKeys->insertItem(i18n("volume up"),"volume up",Key_Right);
  userKeys->connectItem("volume up",this,SLOT(volumeUpSLOT()));

  userKeys->insertItem(i18n("volume down"),"volume down",Key_Left);
  userKeys->connectItem("volume down",this,SLOT(volumeDownSLOT()));

  userKeys->insertItem(i18n("add new program"),"add new program",CTRL+Key_N);
  userKeys->connectItem("add new program",this,SLOT(addChannelSLOT()));

  userKeys->insertItem(i18n("edit active program"),"edit active program",CTRL+Key_E);
  userKeys->connectItem("edit active program",this,SLOT(editChannelSLOT()));

  userKeys->insertItem(i18n("delete active program"),"delete active program",CTRL+Key_D);
  userKeys->connectItem("delete active program",this,SLOT(deleteChannelSLOT()));

  userKeys->insertItem(i18n("video clip"),"video clip",Key_V);
  userKeys->connectItem("video clip",this,SLOT(videoClipSLOT()));

  userKeys->insertItem(i18n("capture toggle"),"capture toggle",Key_C);
  userKeys->connectItem("capture toggle",this,SLOT(captureToggleSLOT()));

  userKeys->insertItem(i18n("snap shot"),"snap shot",Key_S);
  userKeys->connectItem("snap shot",this,SLOT(screenShotSLOT()));

  userKeys->insertItem(i18n("mute toggle"),"mute toggle",Key_M);
  userKeys->connectItem("mute toggle",this,SLOT(volMuteChangedSLOT()));

  userKeys->insertItem(i18n("fix aspect ratio"),"fix aspect ratio",Key_A);
  userKeys->connectItem("fix aspect ratio",this,SLOT(adjustAspectRatioSLOT()));

  userKeys->insertItem(i18n("full screen toggle"),"full screen toggle",Key_F);
  userKeys->connectItem("full screen toggle",this,SLOT(fullScreenSLOT()));

  userKeys->insertItem(i18n("toggle always on top"),"toggle always on top",CTRL+Key_A);
  userKeys->connectItem("toggle always on top",this,SLOT(alwaysOnTopSLOT()));

  userKeys->insertItem(i18n("general setup"),"general setup",ALT+Key_G);
  userKeys->connectItem("general setup",this,SLOT(setupSLOT()));

  userKeys->insertItem(i18n("key bindings"),"key bindings",ALT+Key_K);
  userKeys->connectItem("key bindings",this,SLOT(userKeysConfigSLOT()));

  userKeys->insertItem(i18n("show program browser"),"toggle program box",Key_B);
  userKeys->connectItem("toggle program box",this,SLOT(toggleChannelBoxSLOT()));

//    userKeys->insertStdItem(KAccel::Quit, i18n("exit kwintv"));
//    userKeys->connectItem(KAccel::Quit,this,SLOT(closeRequestSLOT()));

  userKeys->insertItem(i18n("exit kwintv"),"exit",Key_Q);
  userKeys->connectItem("exit",this,SLOT(closeRequestSLOT()));

  userKeys->insertItem(i18n("toggle all Menus"),"toggle all Menus",Key_T);
  userKeys->connectItem("toggle all Menus",this,SLOT(toggleTVonlySLOT()));

  userKeys->insertItem(i18n("show info"),"show info",Key_T);
  userKeys->connectItem("show info",this,SLOT(showInfoSLOT()));

  userKeys->readSettings(); // finally, read the settings done by the user

  // irc stuff
#ifdef HAVE_IRC
  ircCodes = new KIRCWidget( kapp->config() );
  ircCodes->registerWidget("wintvmain",this);

  // function buttons
  ircCodes->addCode("previous channel","PREV");
  ircCodes->connectFunction("wintvmain","previous channel",this,SLOT(channelPreviousSLOT()));

  ircCodes->addCode("program up","CH+", KCodeEntry::normal, KCodeEntry::often);
  ircCodes->connectFunction("wintvmain","program up",this,SLOT(channelDownSLOT()));
  ircCodes->addCode("program down","CH-", KCodeEntry::normal, KCodeEntry::often);
  ircCodes->connectFunction("wintvmain","program down",this,SLOT(channelUpSLOT()));
  ircCodes->addCode("volume up","VOL+",KCodeEntry::normal,KCodeEntry::often);
  ircCodes->connectFunction("wintvmain","volume up",this,SLOT(volumeUpSLOT()));
  ircCodes->addCode("volume down","VOL-",KCodeEntry::normal,KCodeEntry::often);
  ircCodes->connectFunction("wintvmain","volume down",this,SLOT(volumeDownSLOT()));
  ircCodes->addCode("snap shot","RESERVED");
  ircCodes->connectFunction("wintvmain","snap shot",this,SLOT(screenShotSLOT()));
  ircCodes->addCode("mute","MUTE");
  ircCodes->connectFunction("wintvmain","mute",this,SLOT(volMuteChangedSLOT()));
  ircCodes->addCode("full screen","FULL_SCREEN");
  ircCodes->connectFunction("wintvmain","full screen",this,SLOT(fullScreenSLOT()));
  ircCodes->addCode("capture toggle","TV");
  ircCodes->connectFunction("wintvmain","capture toggle",this,SLOT(captureToggleSLOT()));
  ircCodes->addCode("show info","SOURCE");
  ircCodes->connectFunction("wintvmain","show info",this,SLOT(showInfoSLOT()));
  ircCodes->addCode("exit kwintv","MINIMIZE");
  ircCodes->connectFunction("wintvmain","exit kwintv",this,SLOT(closeRequestSLOT()));

  // number buttons
  ircCodes->addCode( "0",  "0", KCodeEntry::number );
  ircCodes->addCode( "1",  "1", KCodeEntry::number );
  ircCodes->addCode( "2",  "2", KCodeEntry::number );
  ircCodes->addCode( "3",  "3", KCodeEntry::number );
  ircCodes->addCode( "4",  "4", KCodeEntry::number );
  ircCodes->addCode( "5",  "5", KCodeEntry::number );
  ircCodes->addCode( "6",  "6", KCodeEntry::number );
  ircCodes->addCode( "7",  "7", KCodeEntry::number );
  ircCodes->addCode( "8",  "8", KCodeEntry::number );
  ircCodes->addCode( "9",  "9", KCodeEntry::number );
  ircCodes->addCode("10", "10", KCodeEntry::number );

  // special buttons
  ircCodes->addCode("+10", "TIME",   KCodeEntry::up10 );
  ircCodes->addCode("+20", "REPEAT", KCodeEntry::up20 );
  ircCodes->addCode("+30", "FADER",  KCodeEntry::up30 );
  ircCodes->addCode(">10", ">10",    KCodeEntry::twoDigits );

  connect( ircCodes, SIGNAL(numKeyReceivedSIGNAL(int)),
	   this, SLOT(switchToChannelSLOT(int)));
  connect( ircCodes, SIGNAL(specialKeyReceivedSIGNAL( KCodeEntry::codeType, int )),
	   this, SLOT( displaySpecialKeySLOT( KCodeEntry::codeType, int )));
#endif // HAVE_IRC

  /////////////////////////////////////////////////////////////////////////////
  statusBar()->insertItem(i18n("File:"),SB_PROG_TEXT);
  statusBar()->insertItem("WWWWWWWW",SB_PROG);
  statusBar()->changeItem(rc->channelListName,SB_PROG);
  statusBar()->insertItem(i18n("Snap Shot:"),SB_SNAP_TEXT);
  statusBar()->insertItem("0000",SB_SNAP);
  statusBar()->insertItem(i18n("Volume:"),SB_VOL_TXT);
  statusBar()->insertItem("00%",SB_VOL);
  statusBar()->insertItem(i18n("Size:"),SB_SIZE_TXT);
  statusBar()->insertItem("000x000",SB_SIZE);

  ///////////////////////////////////////////////////////////////////////
  // load channel-file
  channel_list.setAutoDelete ( true );
  loadChannels(rc->dataDir+QString("/")+rc->channelListName);

  /////////////////////////////////////////////////////////////////////////////
  // resize etc.

  //TODO: shouldn't this be handled by session management?
  if ( !rc->ol_allbar ) {
    menuBar()->hide();
    toolBar()->hide();
    statusBar()->hide();
    m_paShowMenuBar->setEnabled( false );
    m_paShowToolBar->setEnabled( false );
    m_paShowStatusBar->setEnabled( false );
  } else {
    if ( !rc->ol_menubar ) menuBar()->hide();
    if ( !rc->ol_toolbar ) toolBar()->hide();
    if ( !rc->ol_statusbar ) statusBar()->hide();
  }
  m_paShowMenuBar->setChecked( rc->ol_menubar && rc->ol_allbar );
  m_paShowToolBar->setChecked( rc->ol_toolbar && rc->ol_allbar );
  m_paShowStatusBar->setChecked( rc->ol_statusbar && rc->ol_allbar );
  m_paHideAll->setChecked( !rc->ol_allbar );

  if ( rc->ol_restore ) {
    resize(rc->ol_geometry.width(),rc->ol_geometry.height());
    move(rc->ol_geometry.x(),rc->ol_geometry.y());
  } else {
    int yoff = (rc->ol_statusbar ? statusBar()->height() : 0)
               + (rc->ol_toolbar ? toolBar()->height()   : 0)
               + (rc->ol_menubar ? menuBar()->height()   : 0);
    resize(400, 300+yoff);
  }
 
  setMinimumSize(160, 120);

  setWFlags(Qt::WResizeNoErase);
  setWFlags(Qt::WRepaintNoErase);
  update();

  /////////////////////////////////////////////////////////////////////////
  // scan command line
  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
  int shift = args->getOption("shift").toInt();
  if (shift < -8192 || shift > 8192) {
    kdWarning() << "argument <shift> not in range -8192 ... 8192" << endl;
    shift = 0;
  }
  int bpp = args->getOption("bpp").toInt();
  if ( bpp != 0 && bpp != 8 && bpp != 15 && bpp != 16 && bpp != 24 && bpp != 32 ) {
    kdWarning() << "argument <bpp> not one of 8, 15, 16, 24, 32" << endl;
    bpp = 0;
  }
  int palette = args->getOption("palette").toInt();
  //int refresh= args->getOption("refresh").toInt();

  rc->bttvDeviceName = args->getOption("device");
  have_vm = args->isSet("xfree");

  /////////////////////////////////////////////////////////////////////////
  // run kv4lsetup
  QString kv4lcall("kv4lsetup -q -l ");
  kv4lcall += rc->bttvDeviceName;
  if ( bpp ) {
    kv4lcall += QString(" -b ") + QString::number(bpp);
  }
  if ( shift ) {
    kv4lcall += QString(" -s ") + QString::number(shift);
  }

  switch (system(kv4lcall.latin1())) {
  case -1:
    kdWarning() << "kwintv: couldn't start kv4lsetup!" << endl;
   break;
  case 0:
   break;
  default:
    kdWarning() << "kwintv: kv4lsetup had some trouble, trying to continue anyway." << endl;
   break;
  }

  /////////////////////////////////////////////////////////////////////////
  // set up winTVScreen widget
  fixed = true;
  if (args->isSet("usexv")) {
     winTVScreen = new WinTVScreen_xv(this, "winTVScreen",
					0/*rc->capture*/,
				    	rc->bttvDeviceName,
				    	bpp, palette);
  } else {
     winTVScreen = new WinTVScreen_v4l(this, "winTVScreen",
					0/*rc->capture*/,
				    	rc->bttvDeviceName,
				    	bpp, palette);
  }


  winTVScreen->setTVPopup( m_pPopupMenu );
  winTVScreen->enableOSDSLOT( OSD_LEVEL_OFF );
  connect(winTVScreen, SIGNAL(fullScreenRequest()), SLOT(fullScreenSLOT()));


  //winTVScreen->setViewRefreshExtra(refresh);
  winTVScreen->setPalette(palette);

  if (rc->alwaysOnTop) {
     KWin::setState(winId(), NET::StaysOnTop);
  }

  setCentralWidget(winTVScreen);

  // set autio mode default
  winTVScreen->changeAudioModeSLOT(VIDEO_SOUND_STEREO);
  int audio = winTVScreen->getAudioMode();

  int i = 0;
  while ( audiomode[i].nr != audio && audiomode[i].nr != -1 ) i++;

  if ( audiomode[i].nr != -1 ) {
    dynamic_cast<KRadioAction*>( m_aModeList.at( i ) )->setChecked( true );
  }

  // does tuner see STEREO ???
//   if ( !winTVScreen->v4lxifP()->capTunerStereo() ) {
//      setItemEnabled (smenu, smid[0],false);
//      setItemEnabled (smenu, smid[2],false);
//      setItemEnabled (smenu, smid[3],false);
//      setItemEnabled (smenu, smid[4],false);
//   }

  // init dcop
//  if ( !kapp->dcopClient()->isAttached() )
    kapp->dcopClient()->attach();

  // check whether we run in a KDE environment
  QByteArray data;
  m_bHaveKDESaver = kapp->dcopClient()->findObject("kdesktop",
                                                   "KScreensaverIface",
                                                   "functions()",
                                                   data,
                                                   m_screenSaverApp,
                                                   m_screenSaverObj);

  if ( m_bHaveKDESaver ) {
    kdDebug() << "Found KDE screen saver" << endl;
    kdDebug() << "DCOP interface: '" << m_screenSaverApp 
              << "/" << m_screenSaverObj << endl;
  } else {
    kdDebug() << "No KDE screen saver found. Assuming standard X saver." 
              << endl;
  }

  // set winTVScreen values
  if ( rc->screenSaverMode == "Always" )
    disableScreenSaverSLOT( true );


  /////////////////////////////////////////////////////////////////////////////
  // set up ctab dialogs
  tab = new ctab(NULL);
  progTab = new c2prog(rc, tab);
  freqTab = new c2freq(rc, tab);
  optsTab = new c2opts(tab);

  tab->addTab(progTab, i18n("&Program"));
  tab->addTab(freqTab, i18n("&Frequency"));
  tab->addTab(optsTab, i18n("&Options"));

  tab->setCancelButton();
  tab->setOkButton();
  tab->setDefaultButton();

  /////////////////////////////////////////////////////////////////////////////
  // set up otab dialogs
  mtab= new otab(NULL, "otab");

  general = new ogeneral(mtab, "gen");
  mtab->addTab(general, i18n("&General"));
  snapshot = new osnapshot(mtab, "ss");
  mtab->addTab(snapshot, i18n("S&nap Shot"));
  fullscreen = new ofullscreen(mtab, "fs", resolution_list);
  mtab->addTab(fullscreen,i18n("&Screen"));
#ifdef HAVE_MIXER
  if (mixer::getNumMixer() > 0) {
    mixdev = new omixdev(rc->mixerDeviceName,winTVScreen->getCanVolume(),mtab);
    mtab->addTab(mixdev, i18n("&Mixer"));
  } else mixdev = NULL;
#endif

  mtab->setOkButton();
  mtab->setCancelButton();
  mtab->setDefaultButton();

  /////////////////////////////////////////////////////////////////////////
  // connect everything
  connect(winTVScreen,SIGNAL(keyPressedSIGNAL(int)),this,SLOT(keyPressedSLOT(int)));
  connect(winTVScreen,SIGNAL(autoDetectAudioModeSIGNAL(int)),this,SLOT(audioModeChangedSLOT(int)));

  connect(winTVScreen,SIGNAL(toggleTVonlySIGNAL()),this,SLOT(toggleTVonlySLOT()));
  connect(this,SIGNAL(channelChangedSIGNAL()),this,SLOT(channelChangedSLOT()));

  // connect capture changing
  connect(this,SIGNAL(captureToggleSIGNAL(int)),winTVScreen,SLOT(changeCaptureSLOT(int)));

  // connect audio
  connect(this,SIGNAL(audioModeChangedSIGNAL(int)),winTVScreen,SLOT(changeAudioModeSLOT(int)));
  connect(this,SIGNAL(audioMuteToggledSIGNAL()),winTVScreen,SLOT(toggleAudioMuteSLOT()));
  // FIX ME FIX ME
  //if (rc->use_tv_volume)
  connect(this,SIGNAL(volumeChangedSIGNAL(int,int)),winTVScreen,SLOT(changeAudioVolumeSLOT(int,int)));

  connect(progTab,SIGNAL(newRangeSIGNAL(int,int,int)),freqTab,SLOT(newRangeSLOT(int,int,int)));
  connect(progTab,SIGNAL(frequencyChangedSIGNAL(int)),freqTab,SLOT(frequencyChangedSLOT(int)));

  // connect x11Events
  connect(this,SIGNAL(x11EventSIGNAL(XEvent *)),winTVScreen,SLOT(Shellx11EventSLOT(XEvent *)));

  // connect ctab dialog apply, cancel, default
  connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyTabSLOT()));
  connect(tab,SIGNAL(cancelButtonPressed()),this,SLOT(cancelTabSLOT()));
  connect(tab,SIGNAL(defaultButtonPressed()),this,SLOT(defaultTabSLOT()));

  // connect cmtab dialog apply, cancel, default
  connect(mtab,SIGNAL(applyButtonPressed()),this,SLOT(applyMtabSLOT()));
  connect(mtab,SIGNAL(cancelButtonPressed()),this,SLOT(cancelMtabSLOT()));
  connect(mtab,SIGNAL(defaultButtonPressed()),this,SLOT(defaultMtabSLOT()));

  // timer for channel change by keyboard
  keyTimer = new QTimer(this);
  connect(keyTimer,SIGNAL(timeout()),this,SLOT(keyTimerSLOT()));
}

void winTVMainWidget::setupActions()
{
  // file menu
  (void) new KAction( i18n( "&New Channels" ), "filenew", 0, this, SLOT(newChannelsSLOT()), actionCollection(), "channels_new" );
  (void) new KAction( i18n( "&Open Channels..."), "fileopen", 0, this, SLOT(loadChannelsSLOT()), actionCollection(), "channels_open" );
  KActionMenu* am = new KActionMenu( i18n("&Import Channels"), actionCollection(), "channels_import" );
  am->insert( new KAction( i18n( "&xtvscreen" ), 0, this, SLOT(importXTVSCREENChannelsSLOT()), actionCollection(), "imp_xtvscreen" ) );
//  am->insert( new KAction( i18n( "x&awtv" ), 0, this, SLOT(importXAWTVChannelsSLOT()), actionCollection(), "imp_xawtv" ) );
  (void) new KAction( i18n( "&Save Channels" ), "filesave", 0, this, SLOT(saveChannelsSLOT()), actionCollection(), "channels_save" );
  (void) new KAction( i18n( "Save Channels &As..." ), 0, this, SLOT(saveChannelsAsSLOT()), actionCollection(), "channels_save_as" );
  (void) KStdAction::quit( kapp, SLOT(quit()), actionCollection(), "quit" );

  // program menu
  m_paProgBrowser = new KToggleAction( i18n( "&Browser..." ), "ktv_cs", 0, this, SLOT(toggleChannelBoxSLOT()), actionCollection(), "prog_browser" );
  if ( rc->ol_ch_wasvis ) m_paProgBrowser->setChecked( true );;

  (void) new KAction( i18n( "&New" ), "filenew", 0, this, SLOT(addChannelSLOT()), actionCollection(), "prog_new" );
  (void) new KAction( i18n( "&Edit" ), "edit", 0, this, SLOT(editChannelSLOT()), actionCollection(), "prog_edit" );
  (void) KStdAction::cut( this, SLOT(deleteChannelSLOT()), actionCollection(), "prog_cut");
  (void) KStdAction::copy( this, SLOT(copyChannelSLOT()), actionCollection(), "prog_copy");
  (void) KStdAction::paste( this, SLOT(pasteChannelSLOT()), actionCollection(), "prog_paste");
  (void) new KAction( i18n( "&Wizard..." ), "wizard", 0, this, SLOT(scanChannelsSLOT()), actionCollection(), "prog_wizard" );

  // television menu
  m_paCapture = new KToggleAction( i18n( "&Capture on" ), "ktv_capture", 0, this, SLOT(captureToggleSLOT()), actionCollection(), "tv_capture" );
  if ( rc->capture ) m_paCapture->setChecked( true );
  m_paFullScreen = new KToggleAction( i18n( "&Full Screen" ), "window_fullscreen", 0, this, SLOT(fullScreenSLOT()), actionCollection(), "tv_fullscreen" );
  m_paSizeMenu = new KSelectAction( i18n("S&et Size"), 0, actionCollection(), "tv_set_size" );
  connect( m_paSizeMenu, SIGNAL(activated(int)), this, SLOT(sizemenuSLOT(int)) );

  int i = 0;
  QStringList sizeList;
  while (cv_dimensions[i].id != -1 ) {
    sizeList.append( cv_dimensions[i].sname );
    i++;
  }

  m_paSizeMenu->setItems( sizeList );

  (void) new KAction( i18n( "Fix &Aspect Ratio" ), "ktv_transform", 0, this, SLOT(adjustAspectRatioSLOT()), actionCollection(), "tv_fix_aspect" );
  (void) new KAction( i18n( "&Snap Shot" ), "ktv_snapshot", 0, this, SLOT(screenShotSLOT()), actionCollection(), "tv_snapshot" );
  (void) new KAction( i18n( "&Video Clip..." ), 0, this, SLOT(videoClipSLOT()), actionCollection(), "tv_videoclip" );
#ifdef HAVE_VTX
  (void) new KAction( i18n( "&VideoText..." ), 0, this, SLOT(vtxSLOT()), actionCollection(), "tv_vtx" );
#endif

  // audio menu
#ifdef HAVE_MIXER
  for ( i = 0; i < mixer::getNumMixer(); i++ ) {
    KToggleAction* ta = new KToggleAction(i18n("Mixer ") + 
                                          QString(mixer::getMixerNameById(i)) +
                                          "...", 0, this, SLOT(mixerSLOT()), 
                                          actionCollection(), 
                                          QString("mixer_%1").arg(i));
    m_aMixerList.append( ta );
  }
  if (m_aMixerList.count() > 0)
    m_aMixerList.append( new KActionSeparator );
#endif
  m_paAudioMute = new KToggleAction( i18n("M&ute Sound"), "ktv_muteon", 0, this, SLOT(volMuteChangedSLOT()), actionCollection(), "audio_mute" );

  QList<KAction> modeList;
  i = 0;
  while (-1 != audiomode[i].nr) {
    KRadioAction* ra = new KRadioAction( audiomode[i].str, 0, actionCollection(), QString( "mode_%1" ).arg( i ) );
    connect( ra, SIGNAL(toggled(bool)), this, SLOT(setAudioModeSLOT(bool)) );
    ra->setExclusiveGroup("audio_mode");
    m_aModeList.append( ra );
    i++;
  }

  // options menu
  m_paShowMenuBar = KStdAction::showMenubar( this, SLOT(toggleMenuBarSLOT()), actionCollection(), "options_show_menubar" );
  m_paShowToolBar = KStdAction::showToolbar( this, SLOT(toggleToolBarSLOT()), actionCollection(), "options_show_toolbar" );
  m_paShowStatusBar = KStdAction::showStatusbar( this, SLOT(toggleStatusBarSLOT()), actionCollection(), "options_show_statusbar" );
  m_paHideAll = new KToggleAction( i18n("Hide All"), 0, this, SLOT(toggleTVonlySLOT()), actionCollection(), "options_hide_all" );
  (void) new KAction( i18n("Always On Top"), "attach", 0, this, SLOT(alwaysOnTopSLOT()), actionCollection(), "options_always_on_top" );
  KStdAction::keyBindings( this, SLOT(userKeysConfigSLOT()), actionCollection(), "options_configure_keybinding" );
#ifdef HAVE_IRC
  (void) new KAction( i18n("Configure &IR Control..."), 0, this, SLOT(ircCodesConfigSLOT()), actionCollection(), "options_configure_irc" );
#endif
  KStdAction::preferences( this, SLOT(setupSLOT()), actionCollection(), "options_configure" );
  (void) new KAction( i18n("&Save Options"), 0, this, SLOT(saveRCSLOT()), actionCollection(), "options_save" );

  // toolbar
  m_paChannelDown = new KAction( i18n("Channel Down"), "back", 0, this, SLOT(channelUpSLOT()), actionCollection(), "channel_down" );
  m_paChannelUp = new KAction( i18n("Channel Up"), "forward", 0, this, SLOT(channelDownSLOT()), actionCollection(), "channel_up" );
  m_paVolDown = new KAction( i18n("Volume Down"), "down", 0, this, SLOT(volumeDownSLOT()), actionCollection(), "audio_vol_down" );
  m_paVolUp = new KAction( i18n("Volume Up"), "up", 0, this, SLOT(volumeUpSLOT()), actionCollection(), "audio_vol_up" );

  createGUI();

  // modifications after GUI has been created
  if ( m_aMixerList.count() > 0 )
    plugActionList( "audio_mixer", m_aMixerList );
  if ( m_aModeList.count() > 0 )
    plugActionList( "audio_modes", m_aModeList );

  getToolBarButton( m_paChannelDown )->setAutoRepeat( true );
  getToolBarButton( m_paChannelUp )->setAutoRepeat( true );
  getToolBarButton( m_paVolDown )->setAutoRepeat( true );
  getToolBarButton( m_paVolUp )->setAutoRepeat( true );

  // channel number lcd
  lcd = new QLCDNumber(toolBar(), "LCD" );
  lcd->setMinimumSize( 1, 1 );
  lcd->setMaximumSize( 99, 99 );

  QColorGroup normal;
  QColorGroup disabled;
  QColorGroup inactive;

  // Set our colours for the normal display
  normal.setColor(QColorGroup::Foreground, QColor(RGBCOLOR(9355867)));
  normal.setColor(QColorGroup::Button, QColor(RGBCOLOR(0)));
  normal.setColor(QColorGroup::Light, QColor(RGBCOLOR(16777215)));
  normal.setColor(QColorGroup::Dark, QColor(RGBCOLOR(6316128)));
  normal.setColor(QColorGroup::Mid, QColor(RGBCOLOR(10789024)));
  normal.setColor(QColorGroup::Text, QColor(RGBCOLOR(16777215)));
  normal.setColor(QColorGroup::Base, QColor(RGBCOLOR(16777215)));

  // Set our colours for the disabled lcd
  disabled.setColor(QColorGroup::Foreground, QColor(RGBCOLOR(8421504)));
  disabled.setColor(QColorGroup::Button, QColor(RGBCOLOR(12632256)));
  disabled.setColor(QColorGroup::Light, QColor(RGBCOLOR(16777215)));
  disabled.setColor(QColorGroup::Dark, QColor(RGBCOLOR(6316128)));
  disabled.setColor(QColorGroup::Mid, QColor(RGBCOLOR(10789024)));
  disabled.setColor(QColorGroup::Text, QColor(RGBCOLOR(8421504)));
  disabled.setColor(QColorGroup::Base, QColor(RGBCOLOR(12632256)));

  // Set our colours for the inactive lcd
  inactive.setColor(QColorGroup::Foreground, QColor(RGBCOLOR(9355867)));
  inactive.setColor(QColorGroup::Button, QColor(RGBCOLOR(12632256)));
  inactive.setColor(QColorGroup::Light, QColor(RGBCOLOR(16777215)));
  inactive.setColor(QColorGroup::Dark, QColor(RGBCOLOR(6316128)));
  inactive.setColor(QColorGroup::Mid, QColor(RGBCOLOR(10789024)));
  inactive.setColor(QColorGroup::Text, QColor(RGBCOLOR(0)));
  inactive.setColor(QColorGroup::Base, QColor(RGBCOLOR(16777215)));
  QPalette widgetPalette(normal, disabled, inactive);
  lcd->setPalette(widgetPalette);

  lcd->setSmallDecimalPoint( false );
  lcd->setNumDigits( 2 );
  lcd->setMode( QLCDNumber::DEC );
  lcd->setSegmentStyle( QLCDNumber::Flat );
  lcd->setFrameStyle( QFrame::NoFrame );

  toolBar()->insertWidget( -1, 22, lcd, 2 );

  // popup
  m_pPopupMenu = new KPopupMenu;
  m_pPopupMenu->setTitle(i18n("KWinTV"));
  m_pPopupMenu->insertItem(i18n("File"), (KPopupMenu*)guiFactory()->container( "file", this ) );
  m_pPopupMenu->insertItem(i18n("Program"), (KPopupMenu*)guiFactory()->container( "prog", this ) );
  m_pPopupMenu->insertItem(i18n("Television"), (KPopupMenu*)guiFactory()->container( "tv", this ) );
  m_pPopupMenu->insertItem(i18n("Audio"), (KPopupMenu*)guiFactory()->container( "audio", this ) );
  m_pPopupMenu->insertItem(i18n("Settings"), (KPopupMenu*)guiFactory()->container( "settings", this ) );
}

// ============================================================================
KToolBarButton* winTVMainWidget::getToolBarButton( KAction* a )
{
assert(a);
  for ( int i = 0; i < a->containerCount(); i++ ) {
    QWidget* tmp = a->container( i );
    QString n = tmp->name();
    if ( n == "mainToolBar" ) {
      return dynamic_cast<KToolBar *>( tmp )->getButton( a->itemId( i ) );
    }
  }
  return 0L;
}

// ============================================================================
void winTVMainWidget::closeRequestSLOT()
{
  if ( (chfile_was_changed && !rc->autosave) ) {
    switch( KMessageBox::warningYesNoCancel ( NULL,
                                      i18n("Some program settings where changed.\n"
				      "Do you want to save them?"),
                                      i18n("KWinTV"),
                                      i18n("&Save"), i18n("&Don't save"), i18n("&Cancel") ) ) {
    case KMessageBox::Yes:
      saveChannels(rc->dataDir+QString("/")+rc->channelListName, false );
      break;
    default:
      return;
      break;
    }
  }

  kapp->quit();

}

// ============================================================================
winTVMainWidget::~winTVMainWidget()
{
#ifdef HAVE_LIBXXF86VM
XLockDisplay(qt_xdisplay());
  int myscreen = XineramaGetScreenForPoint(x()+width()/2,y()+height()/2);
  XF86VidModeLockModeSwitch(qt_xdisplay(), myscreen, 0 );
XUnlockDisplay(qt_xdisplay());
#endif

  // switch back to original mode line
  if ( isfullscreen ) disableFullScreen();

  QPoint win_pos = pos();        // store position
  QSize win_size = size();       // store size
  rc->ol_geometry.setRect(win_pos.x(),win_pos.y(),win_size.width(),win_size.height());
  rc->ol_toolbar = toolBar()->isVisible();
  rc->ol_menubar = menuBar()->isVisible();
  rc->ol_statusbar = statusBar()->isVisible();

  // Make sure to always set the old channel information in
  // the rc file, regardless of the previous contents (removed "if")
  rc->ol_channel = channel_list.at();
  if ( rc->autosave ) saveChannels(rc->dataDir+"/"+rc->channelListName, false );

  // reenable screen saver
  disableScreenSaverSLOT( false );

  delete progTab;
  delete optsTab;
  delete freqTab;
  delete tab;

#ifdef HAVE_MIXER
  if (mixdev)
      delete mixdev;
#endif
  delete snapshot;
  delete general;
  delete mtab;

  //  delete scanwizard;

#ifdef HAVE_IRC
  delete ircCodes;
#endif

  delete winTVScreen;
}

// ============================================================================
void winTVMainWidget::scanChannelsSLOT()
{
  if ( scanwizard == NULL ) scanwizard = new winTVScanWizard;
  scanwizard->setScreen(winTVScreen);
  scanwizard->show();
  scanwizard->reset();

  connect(scanwizard,SIGNAL(endWizardSIGNAL()),this,SLOT(readyScanChannelsSLOT()));
}

// ============================================================================
void winTVMainWidget::readyScanChannelsSLOT()
{
  disconnect(scanwizard);
  scanwizard->hide();

  ChannelList * clist = scanwizard->getChannelListP();

  if ( clist == NULL || clist->count() == 0 ) {
    winTVScreen->setFreq(channel_list.current()->frequency);
    emit channelChangedSIGNAL();
    return;
  }

  channel *ch = clist->at(0);
  while (ch) {
    channel_list.append(ch);
    ch = clist->next();
  }

  channel_list.first();

  emit newChannelListSIGNAL( &channel_list );
  emit channelChangedSIGNAL();
}

// ============================================================================
void winTVMainWidget::audioModeChangedSLOT(int mode)
{
  channel *selchnl = channel_list.current();
  if (mode & AUDIOMODE_AUTODETECT) {
    setCaption(QString(selchnl->name) + i18n(" - (auto) ")
  			     + audiodescr[mode ^ AUDIOMODE_AUTODETECT]);
  } else {
    setCaption(QString(selchnl->name) + " - " + audiodescr[mode]);
  }
}

// ============================================================================
void winTVMainWidget::channelChangedSLOT()
{
  if ( channel_list.at() == -1 ) return;

  channel *ch = channel_list.current();

  setCaption(QString(ch->name) + " - " + audiodescr[winTVScreen->getAudioMode()]);

  lcd->display( channel_list.at()+1 );

  winTVScreen->changeChannelValuesSLOT( ch, channel_list.at() );
}

// ============================================================================
void winTVMainWidget::keyPressedSLOT(int key)
{
  kdWarning() << "main: key pressed in tv screen, key: " <<  key << endl;
}

// ============================================================================
void winTVMainWidget::fullSizeSLOT(int w, int h)
{
    // set new size
    int Yoffset = 0, Xoffset = 0;
    if (toolBar()->isVisible()) {
      if ((toolBar()->barPos() == KToolBar::Left) || (toolBar()->barPos() == KToolBar::Right))
	Xoffset += toolBar()->width();
      if((toolBar()->barPos() == KToolBar::Top) || (toolBar()->barPos() == KToolBar::Bottom))
	Yoffset += toolBar()->height();
    }
    if (menuBar()->isVisible()) {
      Yoffset += menuBar()->height();
    }
    if (statusBar()->isVisible()) {
      Yoffset += statusBar()->height();
    }
    //oldPos=geometry();
    resize(w+Xoffset, h+Yoffset);

    if ( (winTVScreen->height() < h - 4 || winTVScreen->height() > h + 4 ) && !isfullsize ) {
      isfullsize = true;
      fullSizeSLOT(w,h);
    } else {
      isfullsize = false;
    }
}


// ============================================================================
bool winTVMainWidget::isFullScreen() {
  return isfullscreen;
}


// ============================================================================
void winTVMainWidget::fullScreen() {
  if (!isfullscreen) fullScreenSLOT();
}


// ============================================================================
void winTVMainWidget::disableFullScreen() {
  if (isfullscreen) fullScreenSLOT();
}


// ============================================================================
void winTVMainWidget::fullScreenSLOT()
{

  if (!winTVScreen->canFullScreen())
     return;

  isfullscreen = !isfullscreen;

  if (!isfullscreen) { // return from fullscreen
    // old size with frame, menus etc.
    KWin::clearState(winId(), NET::StaysOnTop);

    // set old size & position
    KWin::setType(winId(), NET::Normal);
    channel *ch = channel_list.current();
    showNormal();
    move(old_win_pos);
    resize(old_win_size);
    setCaption(QString(ch->name) + " - " + audiodescr[winTVScreen->getAudioMode()]);
    KWin::setOnAllDesktops(winId(), wasOnAllDesktops);

#ifdef HAVE_LIBXXF86VM
XLockDisplay(qt_xdisplay());
     if (have_vm) {
        int myscreen = XineramaGetScreenForPoint(x()+width()/2,y()+height()/2);
        XF86VidModeLockModeSwitch(qt_xdisplay(),myscreen, 0 );
        if (rc->fs_switch_res && res_was_switched) {
           XF86VidModeSwitchToMode(qt_xdisplay(),
                                   myscreen,
                                   vm_modelines[0]);
           res_was_switched = false;
        }
     }
XUnlockDisplay(qt_xdisplay());
#endif

    winTVScreen->enableOSDSLOT( rc->fs_infooverlay );

    m_paHideAll->setEnabled( true );

    if ( !m_paHideAll->isChecked() ) {
      m_paShowMenuBar->setEnabled( true );
      m_paShowToolBar->setEnabled( true );
      m_paShowStatusBar->setEnabled( true );
      if ( m_paShowMenuBar->isChecked() ) menuBar()->show();
      if ( m_paShowToolBar->isChecked() ) toolBar()->show();
      if ( m_paShowStatusBar->isChecked() ) statusBar()->show();
    }

    updateRects();

    winTVScreen->fullScreenMode(false);

    m_paFullScreen->setChecked( false );
    m_paSizeMenu->setEnabled( true );

    // reenable screen saver
    if ( rc->screenSaverMode == "Fullscreen" )
      disableScreenSaverSLOT( false );

  } else { // go fullscreen

    if ( rc->screenSaverMode == "Fullscreen" )
      disableScreenSaverSLOT( true );

    int wdelta = frameGeometry().width()-width();    // ignore the border
    int hdelta = frameGeometry().height()-height()+wdelta;  // ignore the border+title
    channel *ch = channel_list.current();
    setCaption(QString(ch->name) + " - " + audiodescr[winTVScreen->getAudioMode()]);

    wasOnAllDesktops = KWin::info(winId()).state & 0x02;

    KWin::setType(winId(), NET::Override);
    KWin::setState(winId(), NET::StaysOnTop);
    raise();
    old_win_pos = pos();       // store position
    old_win_size = size();       // store size
    showFullScreen();
    toolBar()->hide();
    menuBar()->hide();
    statusBar()->hide();

    winTVScreen->enableOSDSLOT( OSD_LEVEL_2 );

#ifdef HAVE_LIBXXF86VM
XLockDisplay(qt_xdisplay());
    int i;
    int dx = 0xffffff, dy = 0xffffff, mlx = 0, mly = 0;
    int dxs = 0xffffff, dys = 0xffffff;

    if (have_vm) {
      if ( rc->fs_res < vm_count ) {
	mlx = rc->fs_res;
	dxs = vm_modelines[mlx]->hdisplay - winTVScreen->win_max_width();
	dx = abs(dxs);
	dys = vm_modelines[mlx]->vdisplay - winTVScreen->win_max_height();
	dy = abs(dys);
      } else {
	// get the number of best matching modeline
	for (i = 0; i < vm_count; i++) {
	  if ( abs(vm_modelines[i]->hdisplay - winTVScreen->win_max_width()) < dx ) {
	    dxs = vm_modelines[i]->hdisplay - winTVScreen->win_max_width();
	    dx = abs(dxs);
	    mlx = i;
	  }
	  if ( abs(vm_modelines[i]->vdisplay - winTVScreen->win_max_height() < dy) ) {
	    dys = vm_modelines[i]->vdisplay - winTVScreen->win_max_height();
	    dy = abs(dys);
	    mly = i;
	  }
	}

	mlx = (dx < dy)?mlx:mly;

      }

      res_was_switched = false;

      if (rc->fs_switch_res && mlx != 0) {
        int myscreen = XineramaGetScreenForPoint(x()+width()/2,y()+height()/2);
	XF86VidModeSwitchToMode(qt_xdisplay(),myscreen,vm_modelines[mlx]);
	res_was_switched = true;
      } else {
	res_was_switched = false;
      }
    }
XUnlockDisplay(qt_xdisplay());
#endif // HAVE_LIBXXF86VM

    int myscreen = XineramaGetScreenForPoint(x()+width()/2,y()+height()/2);
    int _x,_y,_w,_h;
    XineramaGetScreenGeometry(myscreen, &_x, &_y, &_w, &_h);
    if (rc->fs_upper_left) {
      setGeometry(_x,_y,winTVScreen->win_max_width(),winTVScreen->win_max_height());
    } else {
      setGeometry(_x,_y,_w,_h+hdelta);
    }

#ifdef HAVE_LIBXXF86VM
XLockDisplay(qt_xdisplay());
    QPoint tvwidget_pos(0,0);

    if (have_vm && res_was_switched) {
      XF86VidModeLockModeSwitch(qt_xdisplay(),myscreen, 1);

      XF86VidModeSetViewPort(qt_xdisplay(),myscreen,
			     ((_w-winTVScreen->win_max_width()-dxs)/2)-tvwidget_pos.x()+2*wdelta,
			     tvwidget_pos.y()+(_h-winTVScreen->win_max_height()-dys)/2);
    }
XUnlockDisplay(qt_xdisplay());
#endif // HAVE_LIBXXF86VM

    if (XineramaGetScreenForPoint(QCursor::pos().x(), QCursor::pos().y()) == myscreen) {
      QCursor::setPos(_x+_w/2,_y+_h/2);
    }

    m_paFullScreen->setChecked( true );
    m_paSizeMenu->setEnabled( false );

    m_paHideAll->setEnabled( false );
    m_paShowMenuBar->setEnabled( false );
    m_paShowToolBar->setEnabled( false );
    m_paShowStatusBar->setEnabled( false );

#ifdef HAVE_LIBXXF86VM
XLockDisplay(qt_xdisplay());
    winTVScreen->fullScreenMode(true,
		tvwidget_pos.x()+(_w-winTVScreen->win_max_width()-dxs)/2,
		tvwidget_pos.y()+(_h-winTVScreen->win_max_height()-dys)/2,
		vm_modelines[mlx]->hdisplay,
		vm_modelines[mlx]->vdisplay, wdelta, hdelta, myscreen);
XUnlockDisplay(qt_xdisplay());
#endif // HAVE_LIBXXF86VM

  } // if (!isfullscreen)

}

// ============================================================================
void winTVMainWidget::adjustAspectRatioSLOT()
{
  if (!isVisible() ) return;

  fixed = true;

  int xa, ya;
  int Xoffset = 0, Yoffset = 0;

  switch (rc->aspectRatio)
    {
    case ASPECT_4_3:
      xa = 4; ya = 3;
      break;
    case ASPECT_16_9:
      xa = 16; ya = 9;
      break;
    default:
      kdWarning() << "main: unsupported value for aspect ratio: " << rc->aspectRatio << endl;
      return;
    }

  if (menuBar()->isVisible()) {
    Yoffset += menuBar()->height();
  }

  if (statusBar()->isVisible()) {
    Yoffset += statusBar()->height();
  }

  if ((toolBar()->barPos() != KToolBar::Floating) && (toolBar()->isVisible())) {
    if ((toolBar()->barPos() == KToolBar::Left) || (toolBar()->barPos() == KToolBar::Right))
      Xoffset += toolBar()->width();
    if((toolBar()->barPos() == KToolBar::Top) || (toolBar()->barPos() == KToolBar::Bottom))
      Yoffset += toolBar()->height();
  }

  // aktuelle Breite u Hoehe der tv screen
  QRect size = winTVScreen->geometry();

  int h = (ya*(size.width()))/xa;
  int w = (xa*(size.height()))/ya;
  int sh = size.height();
  int sw = size.width();

  switch (rc->fixar) {
  case ASPECT_FIX_BOTH:
    if (sh < h) {
      size.setHeight(sh + ((h - sh) / 2) );
      size.setWidth(xa*(size.height())/ya);
      resize(size.width()+Xoffset,size.height()+Yoffset);
    } else if (sw < w) {
	size.setWidth(sw + ((w - sw) / 2) );
	size.setHeight((ya*(size.width()))/xa);
	resize(size.width()+Xoffset,size.height()+Yoffset);
      }
    break;
  case ASPECT_FIX_VER: // set new height
    if (sh != h) {
      size.setHeight((ya*(size.width()))/xa);
      resize(size.width()+Xoffset,size.height()+Yoffset);
    }
    break;
  case ASPECT_FIX_HOR: // set new width
    if (sw != w) {
      size.setWidth(xa*(size.height())/ya);
      resize(size.width()+Xoffset,size.height()+Yoffset);
    }
    break;
  default:
    kdWarning() << "main: Unsupported aspect ratio fix " << rc->autoar << endl;
  }
}

// ============================================================================
void winTVMainWidget::sizemenuSLOT(int id)
{
//  kdDebug() << "winTVMainWidget::sizemenuSLOT start id: " << id << endl;
//   if (cv_dimensions[id].id == -1 ) {
//     rc->border=!rc->border;
//     setItemChecked(sizemenu, sid[0], rc->border );
//   } else {
    m_bPredefSizeSet = true;
    fullSizeSLOT( cv_dimensions[id].w, cv_dimensions[id].h );
    m_bPredefSizeSet = false;
//  kdDebug() << "winTVMainWidget::sizemenuSLOT end" << endl;
//   }
}

// ============================================================================
void winTVMainWidget::toggleChannelBoxSLOT()
{
  if ( !rc->autoplaceChannelBox ) {
    emit toggleChannelBoxSIGNAL( NULL );
    return;
  }

  QPoint to;
  QRect rect = frameGeometry();

  switch ( rc->posOfChannelBox ) {
  case UPPER_LEFT:
    emit toggleChannelBoxSIGNAL( NULL );
    break;
  case LOWER_LEFT:
    emit toggleChannelBoxSIGNAL( NULL );
    break;
  case UPPER_RIGHT:
    to.setX(rect.width()+rect.x()+5);
    to.setY(rect.y());
    emit toggleChannelBoxSIGNAL( &to );
    break;
  case LOWER_RIGHT:
    emit toggleChannelBoxSIGNAL( NULL );
    break;
  default:
    emit toggleChannelBoxSIGNAL( NULL );
  }
}

// ============================================================================
void winTVMainWidget::mousecursorToggleSLOT()
{
  winTVScreen->toggleMouseCursorHidingSLOT();
  rc->automouse = winTVScreen->getCursorHidingState();
//  setItemChecked(optmenu, omid[ID_HIDEMOUSE], rc->automouse); //TODO
}

// ============================================================================
void winTVMainWidget::captureToggleSLOT()
{
  capture = !capture;
//  setItemChecked(tvmenu,tvmid[ID_TVCAPTURE],capture); //TODO
  emit captureToggleSIGNAL(capture);
}

// ============================================================================
void winTVMainWidget::screenShotSLOT()
{
  winTVScreen->snapShotSLOT(rc->ss_prefix+PICNAME,
			    rc->ss_fixed,
			    rc->ss_width,rc->ss_height,
			    rc->ss_format,rc->ss_quality);
  QString number;
  number.setNum(winTVScreen->getShotNumber()+1);
  statusBar()->changeItem(number,SB_SNAP);
}

// ============================================================================
void winTVMainWidget::videoClipSLOT()
{
  emit toggleVClipSIGNAL();
}

// ============================================================================
void winTVMainWidget::setupSLOT()
{
  //if (!rc->sound_on) soundSLOT();
#ifdef HAVE_MIXER
  if (mixdev) mixdev->setData(rc);
#endif
  snapshot->setData(rc);
  general->setData(rc);
  fullscreen->setData(rc);
  mtab->show();
}

// ============================================================================
void winTVMainWidget::userKeysConfigSLOT()
{
  KKeyDialog::configureKeys( userKeys );
}

// ============================================================================
void winTVMainWidget::ircCodesConfigSLOT()
{
#ifdef HAVE_IRC
  ircCodes->configureCodes(this);
#endif
}

// ============================================================================
void winTVMainWidget::mixerSLOT()
{
  kdDebug() << "winTVMainWidget::mixerSLOT " << sender()->name() << endl;
  QString s = sender()->name();
  emit toggleMixerBoxSIGNAL( s.right( s.length() - s.findRev( '_' ) - 1 ).toInt() );
}

// ============================================================================
void winTVMainWidget::setAudioModeSLOT( bool on )
{
  kdDebug() << "winTVMainWidget::setAudioModeSLOT " << sender()->name() << "," << on << endl;
  if ( on ) {
    QString s = sender()->name();
    int id = s.right( s.length() - s.findRev( '_' ) - 1 ).toInt();
    kdDebug() << "switching to: " << audiomode[id].nr << " " << audiomode[id].str << endl;
    emit audioModeChangedSIGNAL( audiomode[id].nr );

    // check result and adjust actions
    int mode = winTVScreen->getAudioMode();
    int i = 0;
    while ( audiomode[i].nr != mode && audiomode[i].nr != -1 ) i++;

    if ( audiomode[i].nr != -1 ) {
      dynamic_cast<KRadioAction*>( m_aModeList.at( i ) )->setChecked( true );
    }
  }
}

// ============================================================================
void winTVMainWidget::addChannelSLOT()
{
  if ( !channel_list.isEmpty() ) {
    progTab->setData( channel_list.current() );
    freqTab->setData( channel_list.current() );
    optsTab->setData( channel_list.current() );
  } else {
    progTab->setData(NULL);
    freqTab->setData(NULL);
    optsTab->setData(NULL);
  }
  //if (!winTVScreen->getCaptureState()) captureToggleSLOT();

  setEnabled(false);

  wasEdit=false;
  tab->setCaption(i18n("KWinTV - Add a new program"));
  tab->show();
}

// ============================================================================
void winTVMainWidget::editChannelSLOT()
{
  //if (!winTVScreen->getCaptureState()) captureToggleSLOT();

  progTab->setData( channel_list.current() );
  freqTab->setData( channel_list.current() );
  optsTab->setData( channel_list.current() );

  setEnabled(false);

  wasEdit = true;
  tab->setCaption(i18n("KWinTV - Edit program")+ " <" + channel_list.current()->name + ">");
  tab->show();
}

// ============================================================================
void winTVMainWidget::deleteChannelSLOT()
{
  channel *newchannel;

  if ( channel_list.isEmpty() ) return;

  // copy the channel
  if ( lastCutCopy != 0 ) { delete lastCutCopy; lastCutCopy=0; }
  lastCutCopy = new channel(*channel_list.current());
  channel_list.remove();
  copyChannel=false;

  if ( channel_list.isEmpty() ) {
    newchannel = new channel(rc->freq, "<no name>",
	                     rc->col, rc->bri, rc->hue, 
                             rc->contr, rc->norm, rc->in, 
                             -1,"??");
    channel_list.append( newchannel );
  }

  emit newChannelListSIGNAL( &channel_list );
  emit channelChangedSIGNAL();
}

// ============================================================================
void winTVMainWidget::copyChannelSLOT()
{
  if ( lastCutCopy != 0 ) { delete lastCutCopy; lastCutCopy=0; }
  lastCutCopy = new channel(*channel_list.current());
  copyChannel = true;
}

// ============================================================================
void winTVMainWidget::pasteChannelSLOT()
{

  if ( lastCutCopy == 0 ) return;

  channel *cfrom = new channel(*lastCutCopy);
  channel *cto = channel_list.current();

  //if (!copyChannel) channel_list.remove( channel_list.find(cfrom) );
  channel_list.insert( channel_list.find(cto), cfrom );

  emit newChannelListSIGNAL( &channel_list );
  emit channelChangedSIGNAL();
}

// ============================================================================
void winTVMainWidget::channelBoxDownSLOT()
{
  kdDebug() << "winTVMainWidget::channelBoxDownSLOT" << endl;
  m_paProgBrowser->setChecked( false );
}

// ============================================================================
void winTVMainWidget::mixerDown( int id )
{
  kdDebug() << "winTVMainWidget::mixerDown" << endl;
  dynamic_cast<KToggleAction*>( m_aMixerList.at( id ) )->setChecked( false );
}

// ============================================================================
void winTVMainWidget::cancelTabSLOT()
{
  setEnabled(true);
  emit channelChangedSIGNAL();
}

// ============================================================================
void winTVMainWidget::defaultTabSLOT()
{
  if ( !channel_list.isEmpty() ) {
    progTab->setData( channel_list.current() );
    freqTab->setData( channel_list.current() );
    optsTab->setData( channel_list.current() );
  } else {
    //options->setDataSLOT(channel_list.current());
  }

  // call winTVscreen-Widget with original settings
  emit channelChangedSIGNAL();
}

// ============================================================================
void winTVMainWidget::applyTabSLOT()
{
  bool waschanged = false;
  chfile_was_changed = true;

  setEnabled(true);

  if (wasEdit) {
    channel *selchnl = channel_list.current();
    int act = channel_list.at();
    channel_list.at(act);

    // copy new values into channel
    progTab->getData(selchnl);
    freqTab->getData(selchnl);
    optsTab->getData(selchnl);

    waschanged = true;

    if (waschanged) emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();
  } else {
    // add new channel
    channel *selchnl = new channel;

    // copy new values into channel
    progTab->getData(selchnl);
    freqTab->getData(selchnl);
    optsTab->getData(selchnl);

    if (channel_list.isEmpty()) {
      channel_list.append(selchnl);
    } else {
      int act = channel_list.at();
      channel_list.at(act);
      if ( !channel_list.insert( channel_list.at(), selchnl ))
         kdWarning() << "main: channel_list.insert == false" << endl;
    }

    emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();
  }
}

// ============================================================================
void winTVMainWidget::applyMtabSLOT()
{
#ifdef HAVE_MIXER
  // omixdev
  if (mixdev) {
    rc->volume_right = mixdev->getData()->volume_right;
    rc->use_tv_volume = mixdev->getData()->use_tv_volume;
    rc->use_master_volume = mixdev->getData()->use_master_volume;
    rc->mixerDevice = mixdev->getData()->mixerDevice;
    rc->focusmute = mixdev->getData()->focusmute;
  }
#endif
  // oscreenshot
  rc->ss_fixed = snapshot->getData()->ss_fixed;
  rc->ss_width = snapshot->getData()->ss_width;
  rc->ss_height = snapshot->getData()->ss_height;
  rc->ss_format = snapshot->getData()->ss_format;
  rc->ss_prefix = QString(snapshot->getPrefix());
  rc->ss_quality = snapshot->getData()->ss_quality;
  // oaspectratio
  rc->autoar = fullscreen->getData()->autoar;
  rc->fixar = fullscreen->getData()->fixar;
  rc->aspectRatio = fullscreen->getData()->aspectRatio;
  // ogeneral
  rc->capture = general->getData()->capture;
  rc->autosave = general->getData()->autosave;
  rc->screenSaverMode = general->getData()->screenSaverMode;
  rc->ol_restore = general->getData()->ol_restore;
  rc->posOfChannelBox = general->getData()->posOfChannelBox;
  rc->autoplaceChannelBox = general->getData()->autoplaceChannelBox;
  rc->ol_channel = general->getData()->ol_channel;
  rc->ol_tooltips = general->getData()->ol_tooltips;
  rc->ol_allowdocking = general->getData()->ol_allowdocking;
  // fullscreen
  rc->fs_res = fullscreen->getData()->fs_res;
  rc->fs_infooverlay = fullscreen->getData()->fs_infooverlay;
  rc->fs_switch_res = fullscreen->getData()->fs_switch_res;

  winTVScreen->enableOSDSLOT( rc->fs_infooverlay );

  if ( rc->ol_allowdocking ) {
    if (!dock_widget) {
       dock_widget = new KDE2DockWidget(this, "KDE2DockWidget");
       connect(dock_widget, SIGNAL(fullScreenRequest()), this, SLOT(fullScreenSLOT()));
       connect(dock_widget, SIGNAL(channelsRequest()), this, SLOT(toggleChannelBoxSLOT()));
       dock_widget->dock();
    }
  } else {
    if (dock_widget) {
       delete dock_widget;
       dock_widget = NULL;
    }
  }

  // apply screen saver changes
  disableScreenSaverSLOT( false );
  if ( rc->screenSaverMode == "Always" ) {
    disableScreenSaverSLOT( true );
  } else if ( rc->screenSaverMode == "Fullscreen" && isfullscreen ) {
    disableScreenSaverSLOT( true );
  } else if ( rc->screenSaverMode == "WindowVisible" && isVisible() ) {
    disableScreenSaverSLOT( true );
  }

}

// ============================================================================
void winTVMainWidget::defaultMtabSLOT()
{
#ifdef HAVE_MIXER
  if (mixdev)
    mixdev->setData(rc);
#endif
  snapshot->setData(rc);
  general->setData(rc);
  fullscreen->setData(rc);
}

// ============================================================================
void winTVMainWidget::cancelMtabSLOT()
{
}

// ============================================================================
void winTVMainWidget::saveChannelsSLOT()
{
  saveChannels(rc->dataDir+"/"+rc->channelListName, false );
}

// ============================================================================
void winTVMainWidget::saveChannelsAsSLOT()
{
  QString fileName =
    KFileDialog::getSaveFileName(rc->dataDir,"*.ch",this,filename);
  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);
	// Make sure that we use a new KConfig object!
    saveChannels( filename, true );
    rc->channelListName=fileName.mid(fileName.findRev('/')+1,fileName.length());
    statusBar()->changeItem(rc->channelListName,SB_PROG);
  }
}

// ============================================================================
void winTVMainWidget::loadChannelsSLOT()
{
  QString fileName =
    KFileDialog::getOpenFileName(rc->dataDir,"*.ch",this);
  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);
    loadChannels(filename);
    rc->channelListName=fileName.mid(fileName.findRev('/')+1,fileName.length());
    statusBar()->changeItem(rc->channelListName,SB_PROG);
  }
}

// ============================================================================
void winTVMainWidget::importXAWTVChannelsSLOT()
{
  char tag[32], val[100];
  char line[255];
  country cntry;

  QString fileName = KFileDialog::getOpenFileName(QDir::homeDirPath (),"*xawtv",this);

  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);

    QFile fin(filename);

    if (!fin.open(IO_ReadOnly)) {
      kdWarning() << "main: Can't load programs from file " << filename << endl;
      if ( channel_list.isEmpty() ) {
	channel *s = new channel(rc->freq, "<no channel>",
                                 rc->col, rc->bri, rc->hue,
                                 rc->contr, rc->norm, rc->in);
	channel_list.append( s );
      }
      return;
    }

    QTextStream t( &fin );

    channel *n;
    // remove existing channels
    channel_list.clear();

    int thenorm = DNORM;
    int theinput = DIN;

    strcpy(line,t.readLine().latin1());
    while(line[0] != '[' ) {
      if ((strlen(line) == 0) || line[0] == '#' || line[0] == '%') {
	strcpy(line,t.readLine().latin1());
	continue;
      }
      if (2 != sscanf(line," %31[^= ] = %99[^\n]",tag,val)) {
	kdWarning() << "main: xawtv: parse error " << tag << ":" << val << " in " << filename << endl;
      }


      if (0 == strcmp(tag,"freqtab")) {
        cntry.change(val);
      } else if (0 == strcmp(tag,"source")) {
	theinput= atoi(val);
      } else if (0 == strcmp(tag,"norm")) {
	thenorm= atoi(val);
      } else if (0 == strcmp(tag,"fullscreen")) {
	kdWarning() << "main: scan xawtv: ignored tag: " << tag << endl;
      } else if (0 == strcmp(tag,"mixer")) {
	kdWarning() << "main: scan xawtv: ignored tag: " << tag << endl;
      }
      strcpy(line,t.readLine().latin1());
    }

    while(!t.eof()) {
      n = new channel(0,"<no name>",DCOL,DBRI,DHUE,DCONTRAST,thenorm,theinput);

      if (n->name) delete(n->name);
      n->name = new char[strlen(line)+1];
      strncpy(n->name, &line[1], strlen(line)-1);
      n->name[strlen(line)-2]=0;
      strcpy(line, t.readLine().latin1());
      while (!t.eof() && (line[0] != '[')) {
	if ((strlen(line) == 0) || line[0] == '#' || line[0] == '%') {
	  strcpy(line,t.readLine().latin1());
	  continue;
	}
	if (2 != sscanf(line," %31[^= ] = %99[^\n]",tag,val)) {
	  kdWarning() << "main: xawtv: parse error " << tag << ":" << val << " in file: " << filename << endl;
	}
	if (0 == strcmp(tag,"key")) {
	  kdWarning() << "main: scan xawtv: ignored tag: " << tag << endl;
	} else if (0 == strcmp(tag,"capture")) {
	  kdWarning() << "main: scan xawtv: ignored tag: " << tag << endl;
	} else if (0 == strcmp(tag,"source")) {
	  n->input = 0;
	} else if (0 == strcmp(tag,"norm")) {
	  n->norm = 0;
	} else if (0 == strcmp(tag,"channel")) {
	  n->frequency = cntry.cftofreq(val,0);
	} else if (0 == strcmp(tag,"fine")) {
	  kdWarning() << "main: scan xawtv: ignored tag: " << tag << endl;
	} else if (0 == strcmp(tag,"color"))
	  n->color = atoi(val);
	else if (0 == strcmp(tag,"bright"))
	  n->brightness = atoi(val);
	else if (0 == strcmp(tag,"hue"))
	  n->hue = atoi(val);
	else if (0 == strcmp(tag,"contrast"))
	  n->contrast = atoi(val);
	else if (0 == strcmp(tag,"freqtab"))
	  kdWarning() << "main: scan xawtv: ignored tag: " << tag << endl;
	strcpy(line,t.readLine().latin1());
      }

      channel_list.append( n );
    }

    emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();

  }
}

// ============================================================================
void winTVMainWidget::importXTVSCREENChannelsSLOT()
{
  QString fileName = KFileDialog::getOpenFileName(QDir::homeDirPath (),"*.*",this,".xtvrc");
  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);
    loadChannels(filename,true);
  }
}

// ============================================================================
void winTVMainWidget::vtxSLOT()
{
}

// ============================================================================
void winTVMainWidget::naSLOT()
{
  KMessageBox::information( this,
                            i18n("This function is not available in this version!"),
			    i18n("KWinTV") + " " + QString(VERSION));

}

// ============================================================================
void winTVMainWidget::saveRCSLOT()
{
  if ( rc->ol_channel != -1 ) rc->ol_channel = channel_list.at();
  rc->ol_geometry = frameGeometry();
  rc->ol_toolbar = toolBar()->isVisible();
  rc->ol_menubar = menuBar()->isVisible();
  rc->ol_statusbar = statusBar()->isVisible();

  if (rc->save())
    KMessageBox::sorry( this,
		        i18n("KWinTV was not able to save options!"),
		        i18n("KWinTV") + " "+QString(VERSION));
}

// ============================================================================
void winTVMainWidget::toggleMenuBarSLOT()
{
  if ( isfullscreen ) return;

  int offset;
  QRect size=geometry();
  if (menuBar()->isVisible()) {
    offset = -menuBar()->height();
    menuBar()->hide();
  } else {
    menuBar()->show();
    offset = menuBar()->height();
  }
  resize(size.width(),size.height()+offset);
}

// ============================================================================
void winTVMainWidget::toggleStatusBarSLOT()
{
  if ( isfullscreen ) return;

  int offset;
  QRect size = geometry();
  if (statusBar()->isVisible()) {
    offset = -statusBar()->height();
    statusBar()->hide();
  } else {
    statusBar()->show();
    offset = statusBar()->height();
  }
  resize(size.width(),size.height()+offset);
}

// ============================================================================
void winTVMainWidget::toggleToolBarSLOT()
{
  if ( isfullscreen || (toolBar()->barPos() == KToolBar::Floating) ) return;

  if ( (toolBar()->barPos() == KToolBar::Floating) ) return;
  int xoffset = 0, yoffset = 0;
  QRect size = geometry();
  if (toolBar()->isVisible()) {
    if ((toolBar()->barPos() == KToolBar::Left) || (toolBar()->barPos() == KToolBar::Right))
      xoffset -= toolBar()->width();
    if((toolBar()->barPos() == KToolBar::Top) || (toolBar()->barPos() == KToolBar::Bottom))
      yoffset -= toolBar()->height();
    toolBar()->hide();
  } else {
    toolBar()->show();
    if ((toolBar()->barPos() == KToolBar::Left) || (toolBar()->barPos() == KToolBar::Right))
      xoffset += toolBar()->width();
    if((toolBar()->barPos() == KToolBar::Top) || (toolBar()->barPos() == KToolBar::Bottom))
      yoffset += toolBar()->height();
  }
  resize(size.width()+xoffset,size.height()+yoffset);
}

// ============================================================================
void winTVMainWidget::dockSLOT( bool visible )
{
  if ( visible ) {
    if ( capture ) {
      emit captureToggleSIGNAL(capture);
    }
  } else {
    if ( capture ) {
      emit captureToggleSIGNAL(!capture);
    }
  }

}

// ============================================================================
void winTVMainWidget::propertiesSLOT()
{
}

// ============================================================================
void winTVMainWidget::volMuteChangedSLOT()
{
  if ( !externalMute ) {
    emit volMuteChangedSIGNAL( rc->sound_on ); // on==true -> mute
    emit audioMuteToggledSIGNAL();
    m_paAudioMute->setChecked( !rc->sound_on );

    if ( rc->sound_on == m_paAudioMute->isChecked() ) {
      externalMute = true;
    }
  } else
    externalMute = false;
}

// ============================================================================
void winTVMainWidget::volMuteChangedExtSLOT(bool /*on*/)
{
  externalMute = true;
  m_paAudioMute->setChecked( !rc->sound_on );
  emit audioMuteToggledSIGNAL();
}

// ============================================================================
void winTVMainWidget::showInfoSLOT()
{
  if ( isfullscreen || (rc->fs_infooverlay >= OSD_LEVEL_1) )
    winTVScreen->showOSDSLOT(channel_list.current());
}

// ============================================================================
void winTVMainWidget::volumeSLOT(int /*vl*/,int /*vr*/)
{
  QString number;
  number.setNum((rc->volume_left+rc->volume_right)/2);
  number.append("%");
  statusBar()->changeItem(number,SB_VOL);
  if ( isfullscreen || (rc->fs_infooverlay >= OSD_LEVEL_2) ) {
    winTVScreen->showOSDSLOT(i18n("volume -- ")+number);
  }
}

// ============================================================================
void winTVMainWidget::volumeUpSLOT()
{
  int vl = rc->volume_left+MIXER_STEP;
  int vr = rc->volume_right+MIXER_STEP;
  if (vl > MIXER_MAX_VALUE) vl = MIXER_MAX_VALUE;
  if (vr > MIXER_MAX_VALUE) vr = MIXER_MAX_VALUE;
  emit volumeChangedSIGNAL(vr,vl);
}

// ============================================================================
void winTVMainWidget::volumeDownSLOT()
{
  int vl = rc->volume_left - MIXER_STEP;
  int vr = rc->volume_right - MIXER_STEP;
  if (vl < 0) vl = 0;
  if (vr < 0) vr = 0;
  emit volumeChangedSIGNAL(vr,vl);
}

// ============================================================================
void winTVMainWidget::enableChannelUp() { chup_disabled = false; }

// ============================================================================
void winTVMainWidget::channelUpSLOT()
{
  if (chup_disabled) return;
  prevChannel = channel_list.at();
  channel_list.prev();
  if ( channel_list.at() == -1 ) channel_list.last();
  chup_disabled = true;
  QTimer::singleShot(275, this, SLOT(enableChannelUp()));

  emit channelChangedSIGNAL();
}


// ============================================================================
void winTVMainWidget::channelPreviousSLOT()
{
channelPrevious();
}


// ============================================================================
void winTVMainWidget::channelPrevious()
{
unsigned int tmp = channel_list.at();
   channel_list.at(prevChannel);
   prevChannel = tmp;
emit channelChangedSIGNAL();
}


// ============================================================================
void winTVMainWidget::enableChannelDown() { chdown_disabled = false; }

// ============================================================================
void winTVMainWidget::channelUp() {
channelDownSLOT();
}


// ============================================================================
void winTVMainWidget::channelDown() {
channelUpSLOT();
}


// ============================================================================
void winTVMainWidget::channelDownSLOT()
{
  if (chdown_disabled) return;
  prevChannel = channel_list.at();
  channel_list.next();
  if ( channel_list.at() == -1 ) channel_list.first();
  chdown_disabled = true;
  QTimer::singleShot(275, this, SLOT(enableChannelDown()));

  emit channelChangedSIGNAL();
}

// ============================================================================
void winTVMainWidget::newChannelsSLOT()
{
  // remove existing channels
  if ( !channel_list.isEmpty() ) channel_list.clear();

  channel *s = new channel(rc->freq,"<no name>",
			   rc->col,rc->bri,rc->hue,rc->contr,rc->norm,rc->in,
			   -1, "??");
  channel_list.append( s );

  emit newChannelListSIGNAL( &channel_list );
  emit channelChangedSIGNAL();
}

// ============================================================================
void winTVMainWidget::loadChannels(QString file,bool import)
{
  channel *n;
  int ver = 5;	// Set a reasonable default value
  bool postsave = false;
  bool flag = false;

	// Start in "legacy mode" by default
	bool legacy = true;

	// Introduced new channel list class, based on KConfig. See
	// channellist.h and channellist.cpp for details. However, we
	// should still be able to parse the old files (one time) to
	// convert them into the new format so we still need some
	// "legacy" code in this method.

	// WARNING: The channel_list object is created here in several
	// different places to avoid error messages during start-up.
	// (KConfig tries to parse the file during its construction.)

	// "Legacy" code to parse old channel files. If format does
	// not match we assume that we already use the new KConfig format.

  QFile channelfile( file );

	// Check if we're able to read the file can be opened, otherwise
	// add a new empty channel to the channel list and exit.
	
  if (!channelfile.open(IO_ReadOnly)) {
    kdWarning() << "main: Can't load program-file " << file << endl;
		// Build a default Channel List & add an "empty" channel
    channel_list = ChannelList();
    channel *s=new channel(rc->freq,"<no name>",
  		     rc->col,rc->bri,rc->hue,rc->contr,rc->norm,rc->in,
			     -1,"??");
    channel_list.append( s );
    emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();
    return;
  }

	// Create new text stream to parse "legacy" channel files
	// Also make sure that the channel list is empty.

  QTextStream fin(&channelfile);
  if ( !channel_list.isEmpty() ) channel_list.clear();

	// Parse the first line(s) of the channel file to check for
	// the version of the file that is being used at the moment

  QString myline = fin.readLine();
  QString check = myline.left(3);
  if (check.contains("###", true)) {
		 // KWinTV is still using the "old" channel file format
		 // so parse it and do not use the new channel format at
		 // this point.
     check = myline.mid(4, 2);
     ver = check.toInt(&flag);
     if (!flag)
        kdDebug() << "ERROR: Parsing channel file version number failed" << endl;
     if (!ver) ver = CFVER_4;
     postsave = true;
		 legacy = true;
  } else {
  	 // We assume here that KWinTV is already using the new format
		 // so close the file we just opened and leave it to the new
		 // Channel List class to read and parse it, then exit.
		 legacy = false;
		 channelfile.close();
		 channel_list = ChannelList( file, legacy );
		 channel_list.load();
	}

  if ( legacy && import )
    ver= CFVER_XTV;

	// "Legacy" code to parse old channel files. We still can use
	// the new KConfig-based channel list class nevertheless

	if ( legacy ) {
	  // Create a new channel list in legacy mode
		channel_list = ChannelList( file, legacy );
    while (!fin.eof()) {
      n = new channel;
      bool brc = n->load(&fin, ver);
      if (brc)
        channel_list.append( n );
    }

		// To prevent error messages during the conversion of the old file
		// to the new file (during KConfig construction) we first delete the
		// old channel file

		channelfile.remove();
	} // end of legacy code

  // set current channel, determined by a setting in the rc file

  if ( !channel_list.isEmpty() ) {
    if ( (rc->ol_channel == -1 ) || (rc->ol_channel > (int)channel_list.count()) )
      channel_list.first();
    else
      channel_list.at(rc->ol_channel);

    prevChannel = channel_list.at();
    emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();
  }

  // save with newest version enabled, this converts the file into the
	// new KConfig-based channel format

	if ( postsave ) saveChannels( file, false );
}

// ============================================================================

void winTVMainWidget::saveChannels( QString file, bool renewConfig )
{
	// We've switched to the KConfig-based channel list class and
	// file format so we don't need to do much here. If renewConfig is
	// true -> make sure to destroy current KConfig & create a new one
	if ( renewConfig ) channel_list.closeConfig();

	// Since there is a chance we never created the KConfig
	// object, make sure it is created before we save!
	channel_list.saveAs(file);
}

// ============================================================================

void winTVMainWidget::closeEvent( QCloseEvent * )
{
  closeRequestSLOT();
}

// ============================================================================

void winTVMainWidget::resizeEvent( QResizeEvent * theEvent)
{
  //kdDebug() << "winTVMainWidget::resizeEvent m_bPredefSizeSet: " << m_bPredefSizeSet << endl;
  if ( !m_bPredefSizeSet )
    m_paSizeMenu->setCurrentItem( -1 );

  if (rc->autoar) adjustAspectRatioSLOT();

  if ( winTVScreen && isVisible() ) kapp->sendEvent(winTVScreen,theEvent);

  updateRects();

  if ( winTVScreen && statusBar()->isVisible() ) {
    QString number;
    number.sprintf("%dx%d",winTVScreen->width(),winTVScreen->height());
    statusBar()->changeItem(number,SB_SIZE);
  }
}

// ============================================================================

void winTVMainWidget::moveEvent( QMoveEvent * theEvent)
{
  if ( winTVScreen && isVisible() ) kapp->sendEvent(winTVScreen,theEvent);
}

// ============================================================================

void winTVMainWidget::updateRects( )
{
  if (rc->autoar) adjustAspectRatioSLOT();

  if ( winTVScreen && statusBar()->isVisible() ) {
    QString number;
    number.sprintf("%dx%d",winTVScreen->width(),winTVScreen->height());
    statusBar()->changeItem(number,SB_SIZE);
  }
}

// ============================================================================

void winTVMainWidget::saveProperties (KConfig*)
{
  rc->save();
}

// ============================================================================

void winTVMainWidget::readProperties (KConfig*)
{
}


// ============================================================================
void winTVMainWidget::switchToChannel( int chnr ) {
switchToChannelSLOT(chnr-1);
}


// ============================================================================
/* Deceiving - goes to (channel# - 1)  */
void winTVMainWidget::switchToChannelSLOT( int chnr )
{
int pc = channel_list.at();
  if ( chnr < 10 ) {
    prevChannel = pc;
    if ( keyTimer->isActive() ) {
      keyTimer->stop();
      key2 = chnr;
      // switch
      keyTimerSLOT();
    } else {
      key1 = chnr;
      key2 = -1;
      displaySpecialKeySLOT(KCodeEntry::number,key1);
      keyTimer->start(rc->ol_keyTimeOut,true);
    }
  } else if ( (channel_list.count() > (unsigned int)chnr) && channel_list.at(chnr) ) {
    prevChannel = pc;
    emit channelChangedSIGNAL();
    if (!capture) emit captureToggleSLOT();
  }
}

// ============================================================================

void winTVMainWidget::displaySpecialKeySLOT( KCodeEntry::codeType c, int nr)
{
#ifdef HAVE_IRC
  if ( isfullscreen || (rc->fs_infooverlay >= OSD_LEVEL_2) ) {
    QString skey;
    switch (c) {
    case KCodeEntry::up10:
      skey = "+10";
     break;
    case KCodeEntry::up20:
      skey = "+20";
     break;
    case KCodeEntry::up30:
      skey = "+30";
     break;
    case KCodeEntry::twoDigits:
      skey = ">10";
     break;
    case KCodeEntry::number:
    default:
      skey.setNum(nr);
    }
    winTVScreen->showOSDSLOT(skey.local8Bit());
  }
#endif
}

// ============================================================================

void winTVMainWidget::keyPressEvent ( QKeyEvent *keyE )
{
int pc = channel_list.at();

  if ( (keyE->key() >= Key_0) && (keyE->key() <= Key_9) ) {
    prevChannel = pc;
    if ( keyTimer->isActive() ) {
      keyTimer->stop();
      key2 = keyE->key()-Key_0;
      keyTimerSLOT();
    } else {
      key1 = keyE->key()-Key_0;
      key2 = -1;
      displaySpecialKeySLOT(KCodeEntry::number,key1);
      keyTimer->start(rc->ol_keyTimeOut,true);
    }
  } else keyE->ignore();
}

void winTVMainWidget::keyTimerSLOT()
{
  int chnr;

  keyTimer->stop();
  chnr = ((-1 == key2) ? key1 - 1 : key1*10 + key2 - 1);
  if ( (channel_list.count() > (unsigned int)chnr) && channel_list.at(chnr) ) {
    emit channelChangedSIGNAL();
    if (!capture) emit captureToggleSLOT();
  }
}

// ============================================================================

void winTVMainWidget::toggleTVonlySLOT()
{
  if ( isfullscreen ) return;

  rc->ol_allbar = !rc->ol_allbar;
  m_paHideAll->setChecked( !rc->ol_allbar );

  m_paShowMenuBar->setEnabled( rc->ol_allbar );
  m_paShowToolBar->setEnabled( rc->ol_allbar );
  m_paShowStatusBar->setEnabled( rc->ol_allbar );

  int Xoffset = 0, Yoffset = 0;
  QRect size = geometry();
  if (!rc->ol_allbar) {       // hide all bars

    if (menuBar()->isVisible()) {
      Yoffset = menuBar()->height();
      menuBar()->hide();
    }

    if (statusBar()->isVisible()) {
      Yoffset += statusBar()->height();
      statusBar()->hide();
    }

    if (toolBar()->isVisible() && toolBar()->barPos() != KToolBar::Floating) {
      if ((toolBar()->barPos() == KToolBar::Left) || (toolBar()->barPos() == KToolBar::Right))
	Xoffset += toolBar()->width();
      if((toolBar()->barPos() == KToolBar::Top) || (toolBar()->barPos() == KToolBar::Bottom))
	Yoffset += toolBar()->height();
      toolBar()->hide();
    }
    resize(size.width()-Xoffset,size.height()-Yoffset);

  } else {
    // show all bars that where shown before
    if ( m_paShowMenuBar->isChecked() ) menuBar()->show();
    if ( m_paShowToolBar->isChecked() ) toolBar()->show();
    if ( m_paShowStatusBar->isChecked() ) statusBar()->show();

    if ( m_paShowMenuBar->isChecked() ) Yoffset = menuBar()->height();
    if ( m_paShowToolBar->isChecked() ) {
      if ((toolBar()->barPos() == KToolBar::Left) || (toolBar()->barPos() == KToolBar::Right))
	Xoffset += toolBar()->width();
      if((toolBar()->barPos() == KToolBar::Top) || (toolBar()->barPos() == KToolBar::Bottom))
	Yoffset += toolBar()->height();
    }
    if ( m_paShowStatusBar->isChecked() ) Yoffset += statusBar()->height();
    resize(size.width() + Xoffset, size.height() + Yoffset);
  }
}

// ============================================================================
// post init
void winTVMainWidget::emitSetupSignals()
{
  emit newChannelListSIGNAL( &channel_list );
  volumeSLOT(rc->volume_left,rc->volume_right);

  // startup default is muted
  if ( !rc->sound_on ) {
    m_paAudioMute->setChecked( true );
    emit audioMuteToggledSIGNAL();
  } else {

  }

  if (!channel_list.isEmpty()) {
    emit channelChangedSIGNAL();
  }
  winTVScreen->postInit();
  winTVScreen->enableOSDSLOT( rc->fs_infooverlay );

  if (rc->capture) emit captureToggleSIGNAL(capture);
  //winTVScreen->capture(rc->capture);

  if (rc->og_first_time) {
    rc->og_first_time = 0;
    scanChannelsSLOT();
  }
}

// ============================================================================
void winTVMainWidget::dataReceivedSLOT(int /*data*/)
{
}

// ============================================================================
bool winTVMainWidget::x11Event( XEvent *event )
{
  Window wwin;

  if ( (!isVisible()) || (wwin=(Window) winId()) == None ) return(false);

  switch ( event->type ) {
  case MapNotify:
    if ( capture ) {
      emit captureToggleSIGNAL(true);
    }

    winTVScreen->newTVGeometry(false);   // HACK to make undocking work.
                                         // Rant: for some reason when I
                                         // undock (but not unminimize or
                                         // return from another desktop), the
                                         // pos of winTVScreen is off by 4x20
                                         // pixels (left x up).  This is the
                                         // exact size of the titlebar+border.
                                         // Best I can tell, this is a bug in
                                         // QT or KWin.
  case UnmapNotify:
    emit x11EventSIGNAL( event );
    break;
  }
  return (false);

}

// ============================================================================
void winTVMainWidget::alwaysOnTopSLOT()
{
  rc->alwaysOnTop = rc->alwaysOnTop ? false : true;

//   setItemChecked(optmenu, omid[ID_ALWAYSTOP], rc->alwaysOnTop);
  if (rc->alwaysOnTop)
     KWin::setState(winId(), NET::StaysOnTop);
  else
     KWin::clearState(winId(), NET::StaysOnTop);
}

// ============================================================================
void winTVMainWidget::hide() {
    if ( capture ) {
       emit captureToggleSIGNAL(false);
    }
    KMainWindow::hide();
    if (rc->focusmute) {
       emit volMuteChangedSIGNAL(true);
    }
    if (dock_widget) {
    }
}


// ============================================================================
void winTVMainWidget::showEvent(QShowEvent * /*se*/) {
  if (rc->focusmute) emit volMuteChangedSIGNAL(false);
  if ( rc->screenSaverMode == "WindowVisible" )
    disableScreenSaverSLOT( true );
}


// ============================================================================
void winTVMainWidget::hideEvent(QHideEvent * /*he*/) {
  if (isMinimized() && dock_widget)
     hide();     // this makes minimize make the taskbar entry disappear
  else if (rc->focusmute) emit volMuteChangedSIGNAL(true);

  if ( rc->screenSaverMode == "WindowVisible" )
    disableScreenSaverSLOT( false );

   XSetWindowAttributes attributes;
   unsigned long valuemask = CWOverrideRedirect|
                             CWBackingStore|
                             CWSaveUnder|
                             CWBackPixmap;
   attributes.background_pixmap = None;
   attributes.save_under = False;
   attributes.override_redirect = True;
   attributes.backing_store = NotUseful;
   QRect zz = frameGeometry();
   WId rw = XCreateWindow(qt_xdisplay(), qt_xrootwin(), 
                          zz.x(), zz.y(),
                          zz.width()+1, zz.height()+1,
                          0,
                          CopyFromParent, CopyFromParent,
                          CopyFromParent, valuemask,
                          &attributes);
   XMapWindow(qt_xdisplay(), rw);
   XDestroyWindow(qt_xdisplay(), rw);
   XFlush(qt_xdisplay());
}


// ============================================================================
void winTVMainWidget::wheelEvent(QWheelEvent *ev)
{
  int delta = ev->delta();
  delta /= 4;

  if (delta < 0)
    channelUpSLOT();
  else
    channelDownSLOT();
}

// ============================================================================
void winTVMainWidget::show() {
    KMainWindow::show();

    if (rc->focusmute) emit volMuteChangedSIGNAL(false);
}


// ============================================================================
void winTVMainWidget::disableScreenSaverSLOT( bool disable )
{
  if ( m_bHaveKDESaver ) { // KDE screensaver
    // get the current config state
    QString file = locate( "config", "kdesktoprc" );
    KConfig config( file );

    config.reparseConfiguration();
    config.setGroup("ScreenSaver");
    bool e  = config.readBoolEntry("Enabled", false);
    //kdDebug() << "KDE Saver: current status: " << e << endl;

    if ( disable ) { // deactivate screensaver
      //kdDebug() << "KDE Saver: disabling..." << endl;

      // remember start setting
//       m_bKDESAverDefault = e;

      // disable
      QByteArray data;
      QDataStream arg(data, IO_WriteOnly);
      arg << false;
      if ( !kapp->dcopClient()->send( m_screenSaverApp, m_screenSaverObj, "enable(bool)", data ) )
        kdDebug() << "KDE Saver: Failed! There was some error in dcop communication" << endl;
      //else
      //  kdDebug() << "KDE Saver: Ok." << endl;

    } else { // activate screensaver
      //kdDebug() << "KDE Saver: enabling..." << endl;

      if ( e ) {
        // enable
        QByteArray data;
        QDataStream arg(data, IO_WriteOnly);
        arg << true;
        if ( !kapp->dcopClient()->send( m_screenSaverApp, m_screenSaverObj, "enable(bool)", data ) )
          kdDebug() << "KDE Saver: Failed! There was some error in dcop communication" << endl;
        //else
        //  kdDebug() << "KDE Saver: Ok." << endl;
      }
    }
  } else { // legacy X screen saver

#ifdef HAVE_LIBXDPMS
    CARD16 dpms_state;
    int dpms_dummy;
#endif
    int t, i, p, a;
    Display *displ = qt_xdisplay();

    if ( disable ) { // deactivate screensaver
      //kdDebug() << "Legacy X Saver: disabling..." << endl;

#ifdef HAVE_LIBXDPMS
      if ( (DPMSQueryExtension(displ, &dpms_dummy, &dpms_dummy)) && (DPMSCapable(displ)) ) {
        DPMSInfo(displ, &dpms_state, &dpms_on);
        DPMSDisable(displ);
      }
#endif
      XGetScreenSaver(displ,&ss_timeout,&ss_interval,&ss_prefer_blanking,&ss_allow_exposures);
      XSetScreenSaver(displ,0,ss_interval,ss_prefer_blanking,ss_allow_exposures);
    } else { // activate screensaver
      //kdDebug() << "Legacy X Saver: enabling..." << endl;

      XGetScreenSaver(displ,&t,&i,&p,&a);
      if (t == 0 && i == ss_interval && p == ss_prefer_blanking && a == ss_allow_exposures)
        XSetScreenSaver(displ,ss_timeout,ss_interval,ss_prefer_blanking,ss_allow_exposures);
#ifdef HAVE_LIBXDPMS
      if ( (DPMSQueryExtension(displ, &dpms_dummy, &dpms_dummy) ) &&
           (DPMSCapable(displ)) && (dpms_on) ) {
        DPMSEnable(displ);
      }
#endif
    }
  }
}

// ============================================================================

#include "wintvmain.moc"



// local Variables: ***
// compile-command-1: "make kwintv" ***
// compile-command-2: "make" ***
// End: ***

