/*
    kwintv using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)
    Copyright (C) 2001 George Staikos (staikos@kde.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef wintvmainincluded
#define wintvmainincluded

#include <wintv.h>

#include <qpixmap.h>
#include <qpushbt.h>

#include <ctab.h>
#include <c2prog.h>
#include <c2opts.h>
#include <c2freq.h>

#include <otab.h>
#include <omixdev.h>
#include <ogeneral.h>
#include <osnapshot.h>
#include <ofullscreen.h>

#include <kde2dock.h>

#include <kapp.h>
#include <kmainwindow.h>

#include <kaccel.h>        // KDE 1.1
#include <kkeydialog.h>

#include <wintvscreen_v4l.h>
#include <rcfile.h>
#include <channel.h>
#include <wintvscan.h>
#include <videoclip.h>

#include "kwintviface.h"
#include "channellist.h"

#ifdef HAVE_IRC
#include <kirc.h>
#else
#include <kircdata.h>
#endif

class KXv;

#ifdef myDEBUG
#define mainDEBUG
#endif

#define FRAME_NONE 0
#define FRAME_LEFT 1
#define FRAME_RIGHT 2
#define FRAME_UP 3
#define FRAME_DOWN 4

#define UPPER_LEFT  1
#define LOWER_LEFT  2
#define UPPER_RIGHT 3
#define LOWER_RIGHT 4

class KToggleAction;
class KSelectAction;
class KToolBarButton;

class winTVMainWidget : public KMainWindow, virtual public KWinTVIface
{
  Q_OBJECT

public:
  winTVMainWidget( rcfile *_rc=0,
		   QWidget* parent = 0, const char* name = 0 );
  ~winTVMainWidget();

  winTVScreenWidget *winTVScreenPtr() { return winTVScreen; }

  void emitSetupSignals();

protected:
  virtual void showEvent(QShowEvent *se);
  virtual void hideEvent(QHideEvent *he);
  virtual void wheelEvent(QWheelEvent *);

  void setupActions();
  KToolBarButton* getToolBarButton( KAction* a );

public:
  bool isFullScreen();
  void fullScreen();
  void disableFullScreen();
  void switchToChannel( int );
  void channelUp();
  void channelDown();
  void channelPrevious();

public slots:
  void disableScreenSaverSLOT( bool disable );
  void fullScreenSLOT();
  virtual void hide();
  virtual void show();

  // audio & volume
  void audioModeChangedSLOT(int mode);
  void volumeUpSLOT();
  void volumeDownSLOT();
  void volumeSLOT(int vl, int vr);
  void volMuteChangedExtSLOT(bool);

  // channel selection
  void channelUpSLOT();
  void channelDownSLOT();
  void channelChangedSLOT();
  void channelPreviousSLOT();
  void enableChannelUp();
  void enableChannelDown();

  // channel managing
  void addChannelSLOT();
  void editChannelSLOT();
  void deleteChannelSLOT();
  void copyChannelSLOT();
  void pasteChannelSLOT();

  void channelBoxDownSLOT();
  void mixerDown( int id );

protected slots:
  // buttonbar buttons and keys
  void keyPressedSLOT(int key);
  void userKeysConfigSLOT();
  void closeRequestSLOT();

  // irc slots
  //#ifdef HAVE_IRC
  void ircCodesConfigSLOT();
  void displaySpecialKeySLOT( KCodeEntry::codeType, int );
  //#endif

  // channel
  void switchToChannelSLOT( int );

  // menu bar
  void setupSLOT();
  void naSLOT();
  void videoClipSLOT();
  void propertiesSLOT();
  void captureToggleSLOT();
  void screenShotSLOT();

  // audio: volume, mute, audio
  void volMuteChangedSLOT();
  void mixerSLOT();
  void setAudioModeSLOT( bool on );

  // adjustments: aspect ratio and full screen
  void adjustAspectRatioSLOT();
  void sizemenuSLOT(int id);
  void fullSizeSLOT(int w, int h);

  // channel file actions
  void scanChannelsSLOT();
  void readyScanChannelsSLOT();
  void newChannelsSLOT();
  void saveChannelsSLOT();
  void saveChannelsAsSLOT();
  void loadChannelsSLOT();
  void importXAWTVChannelsSLOT();
  void importXTVSCREENChannelsSLOT();

  // ctab
  void defaultTabSLOT();
  void applyTabSLOT();
  void cancelTabSLOT();

  // mtab
  void defaultMtabSLOT();
  void applyMtabSLOT();
  void cancelMtabSLOT();

  // rc
  void saveRCSLOT();

  // menubar, toolbar, mousecursor, statusbar
  void toggleToolBarSLOT();
  void toggleMenuBarSLOT();
  void toggleStatusBarSLOT();
  void mousecursorToggleSLOT();
  void alwaysOnTopSLOT();

  void dockSLOT( bool );

  // info
  void toggleChannelBoxSLOT();
  void toggleTVonlySLOT();

  void vtxSLOT();

  void dataReceivedSLOT(int data);

  void showInfoSLOT();

  bool x11Event( XEvent * );

signals:
  void captureToggleSIGNAL(int);
  void audioModeChangedSIGNAL(int);
  void audioMuteToggledSIGNAL();
  void volumeChangedSIGNAL(int,int);
  void channelChangedSIGNAL();
  void volMuteChangedSIGNAL(bool);
  void newChannelListSIGNAL(ChannelList *clist);
  void toggleChannelBoxSIGNAL( QPoint * );
  void toggleVClipSIGNAL();
  void toggleMixerBoxSIGNAL(int id);
  void x11EventSIGNAL( XEvent * );

private slots:
  void closeEvent ( QCloseEvent *);
  void keyPressEvent ( QKeyEvent * );
  void resizeEvent( QResizeEvent * );
  void moveEvent( QMoveEvent * );
  void updateRects();

  void saveProperties (KConfig*);
  void readProperties (KConfig*);

  void keyTimerSLOT();

private:
  void saveChannels( QString file, bool renewConfig=false );
  void loadChannels( QString file, bool import=false );

private:
  static QList<winTVMainWidget> windowList;

  winTVScreenWidget *winTVScreen;
  rcfile *rc;

  QLCDNumber* lcd;
  QSlider* volslider;

  // actions
  KToggleAction* m_paProgBrowser;
  KToggleAction* m_paCapture;
  KToggleAction* m_paFullScreen;
  QList<KAction> m_aMixerList;
  QList<KAction> m_aModeList;
  KToggleAction* m_paAudioMute;
  KSelectAction* m_paSizeMenu;
  KToggleAction* m_paShowMenuBar;
  KToggleAction* m_paShowToolBar;
  KToggleAction* m_paShowStatusBar;
  KToggleAction* m_paHideAll;
  KAction* m_paChannelDown;
  KAction* m_paChannelUp;
  KAction* m_paVolDown;
  KAction* m_paVolUp;

  KPopupMenu* m_pPopupMenu;

  bool chup_disabled, chdown_disabled;

  // keys & irc codes
  KAccel *userKeys;        // PEG kkeyconf not supported

#ifdef HAVE_IRC
  KIRCWidget *ircCodes;
#endif

  // dialogs
  ctab * tab;
  c2prog * progTab;
  c2opts * optsTab;
  c2freq * freqTab;

  // option dialogs
  otab *mtab;
#ifdef HAVE_MIXER
  omixdev *mixdev;
#endif
  osnapshot *snapshot;
  ogeneral *general;
  ofullscreen *fullscreen;

  // cannel and country stuff
  ChannelList channel_list;
  unsigned int prevChannel;
  channel *lastCutCopy;
  bool copyChannel;

  winTVScanWizard *scanwizard;

  KDE2DockWidget * dock_widget;

  // adjustments: aspect ratio and full screen
  QPoint old_win_pos;
  QSize old_win_size;
  bool isfullscreen,isfullsize;
  bool m_bPredefSizeSet;

  // stuff
  char filename[255];
  char *cnamep;
  int capture;

  QTimer *keyTimer;
  int key1;
  int key2;

  bool fixed,externalMute;
  bool have_vm;
  bool wasEdit;
  bool res_was_switched;
  bool chfile_was_changed;

  bool wasOnAllDesktops;

  // screen saver stuff
  bool m_bHaveKDESaver;
  QCString  m_screenSaverApp, m_screenSaverObj;
  int ss_timeout, ss_interval, ss_prefer_blanking, ss_allow_exposures;
};

#endif

