// -*- c-basic-offset: 2 -*-

/*
    kwintv using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "wintvscan.h"
#include <kapp.h>
#include <klocale.h>


winTVScanWizard::winTVScanWizard(QWidget *parent, const char *name)
  : KWizard(parent,name)
{
  setCaption(i18n("KWinTV ")+VERSION+" - "+i18n("ScanWizard"));

  wizstate = WizCanceled;
  scanwasdone=false;

  wizstart = new wizFinish();
  wiznorm = new wizNorm();
  wizregion = new wizRegion();
  wizscan = new wizScan();
  wizfinish = new wizFinish();

  wizscan->setScreen(screen);

  wizfinish->ShowFinishText();
  wizstart->ShowStartText();

  addPage(wizstart,i18n("Start ScanWizard"));
  addPage(wiznorm,i18n("Select your input norm"));
  addPage(wizregion,i18n("Select your receiving region"));
  addPage(wizscan,i18n("Scan for Programs"));
  addPage(wizfinish,i18n("Finish ScanWizard"));

  setBackEnabled(wizstart,false);
  setNextEnabled(wizfinish,false);
  setFinishEnabled(wizfinish,true);

  connect(this,SIGNAL(selected(const QString &)),this,SLOT(selectedSLOT(const QString &)));
// The following connects the "Finish" button with the okClickedSLOT of the
// wizard. Otherwise the channel data will not go into KWinTV's channel list!!!
  connect(this->finishButton(),SIGNAL(clicked()),this,SLOT(okClickedSLOT()));

  connect(wizscan,SIGNAL(wizScanScanReadySIGNAL()),this,SLOT(scanFinishedSLOT()));

  resize(520,340);

}

winTVScanWizard::~winTVScanWizard()
{
  delete wizstart;
  delete wiznorm;
  delete wizregion;
  delete wizscan;
  delete wizfinish;

  channellist.clear();
}

void winTVScanWizard::reset()
{
  showPage(wizstart);
}

void winTVScanWizard::selectedSLOT(const QString & pagename)
{
  if ( currentPage() != lastpage && (wizScan*)lastpage == wizscan )
    wizscan->wizPBStopSLOT();

  if ( (wizScan*)currentPage() == wizscan ) {
    wizscan->setScanInfo(&channellist,
			 wiznorm->getNorm(),
			 wizregion->getRegion(),
			 wizregion->getScanAll());
  }

  lastpage = currentPage();
}

void winTVScanWizard::okClickedSLOT()
{
  wizscan->wizPBStopSLOT();
  wizstate = WizOk;
  emit endWizardSIGNAL();
}

void winTVScanWizard::cancelClickedSLOT()
{
  wizscan->wizPBStopSLOT();
  channellist.clear();
  wizstate = WizCanceled;
  emit endWizardSIGNAL();
}

void winTVScanWizard::scanFinishedSLOT()
{
  scanwasdone=true;
}

#include "wintvscan.moc"










