/*
 *  Virtual base class for wintvscreen
 *
 *  Copyright (C) 2001 George Staikos <staikos@kde.org>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef WINTVSCREEN_BASE_H
#define WINTVSCREEN_BASE_H

#include <wintv.h>
#include <xrwevent.h>

// includes needed by qt
#include <kapp.h>
#include <qpixmap.h>
#include <qwidget.h>
#include <qtimer.h>
#include <qdatetm.h>
#include <qlcdnum.h>
#include <kpopupmenu.h>

#include <qlabel.h>
#include <qprogressbar.h>

#include <channel.h>
#include <videoclip.h>      // video clip stuff

#include <v4lxif.h>
#include <X11/Xlib.h>

#define WinTVScreen winTVScreenWidget


void flip(char *mem, int len);
XErrorHandler xerror(Display*, XErrorEvent*);


class winTVScreenWidget : public QWidget
{

  Q_OBJECT

public:
  winTVScreenWidget( QWidget *_parent=0, const char *name=0,
		     int _capture=0,
		     const char *_bttvdev=V4L_DEVICE,
		     int _bpp = 0, int _palette = 0,
		     int _freq=DFREQ, int _input=DNORM );
  virtual ~winTVScreenWidget();

  int win_default_width()  { return DEFAULT_WIDTH;  }
  int win_default_height() { return DEFAULT_HEIGHT; }
  int win_max_width()      { return max_width;      }
  int win_max_height()     { return max_height;     }
  virtual int win_width() = 0;
  virtual int win_height() = 0;

  int getCaptureState()      { return captureIsOn;       }
  int getCursorHidingState() { return allowCursorHiding; }

  virtual int getMute() = 0;
  int getAudioMode() { return audioMode; }
  virtual int getVolume() = 0;

  bool canFullScreen();  // true if fullscreen action is allowed
  int getCanVolume() { return audioVolumeCan; }

  void setTVPopup(KPopupMenu *tvp) { tvPopup=tvp; }

  void setViewRefreshMode( bool allow )  { allowViewRefresh = allow;           }
  void setViewRefreshExtra( bool allow ) { allowViewRefreshVisibility = allow; }

  int getViewRefreshMode()      { return allowViewRefresh; }
  int getViewRefreshModeExtra() { return allowViewRefreshVisibility; }

  void setRefreshDelay(int d) { viewRefreshDelay = d;    }
  int getRefreshDelay()       { return viewRefreshDelay; }

  int getShotNumber() { return shotnumber; }

  virtual void setPalette(int p) = 0;

  virtual void fullScreenMode(bool on, int xp=0, int yp=0, int width=0, int height=0, int dw=0, int dh=0, int screen = 0);

  virtual void setFreq(unsigned long freq) { changeFrequencySLOT(freq); }

  // Design problem:  this must be called immediately on startup to set the
  //                  values.  I haven't remove it because it's too much code
  //                  to rework
  static v4lxif *v4lxifP(QString devname=QString::null,
                         int _bpp=0,
                         int _palette=0)
                 { if (!v4lx)
                      v4lx = new v4l1if(devname.latin1(), _bpp, _palette);
                   return v4lx;
                 }

  virtual void newTVGeometry(bool doViewRefresh) = 0;

  // raw* functions don't set any state.  They just do the operation raw on
  // the input device (v4l or xv device)
  virtual void rawSetFreq(unsigned long freq) = 0;
  virtual void rawCapTunerRange(unsigned long *fmin, unsigned long *fmax) = 0;
  virtual void rawSetChannel(int source) = 0;
  virtual void rawSetChannelNorm(int norm) = 0;
  virtual unsigned long rawGetTunerSignal() = 0;

public slots:
  // use this slots to change anything
  virtual void showOSDSLOT(channel *ch) = 0;
  virtual void showOSDSLOT(const char *text ) = 0;
  virtual void enableOSDSLOT( int level );

  // capture
  virtual void changeCaptureSLOT(int value);

  // tuning et al
  virtual void changeFrequencySLOT(int value) = 0;
  virtual void changeBrightnessSLOT(int bri) = 0;
  virtual void changeContrastSLOT(int contr) = 0;
  virtual void changeColorSLOT(int color) = 0;
  virtual void changeHueSLOT(int hue) = 0;
  virtual void changeSourceSLOT(int s) = 0;
  virtual void changeNormSLOT(int n) = 0;

  // set all
  virtual void changeChannelValuesSLOT(channel *c, int id) = 0;

   // sound et al
  virtual void toggleAudioMuteSLOT() = 0;
  virtual void changeAudioVolumeSLOT(int vl, int vr) = 0;
  virtual void changeAudioModeSLOT(int mode) = 0;

  // mouse cursor
  virtual void toggleMouseCursorHidingSLOT();

  // unused stuff
  //virtual void changeTVMultiDisplaySLOT(int x, int y, int w, int h) = 0;

  // one snap shot
  virtual void snapShotSLOT(const char *prefix, int fixed, int width, int height, int format, int quality) = 0;

  // create video clip; format: VIDEOCLIP_AVI, VIDEOCLIP_PPM, VIDEOCLIP_RAW
  //void startVideoClipSLOT(struct MOVIE_PARAMS *par);
  //void endVideoClipSLOT();

  virtual void postInit();

  virtual void Shellx11EventSLOT(XEvent *) = 0;

  /* ----- stop reading --- everything below is private ... --------------- */
private slots:
  virtual void refreshTVActionSLOT() = 0;
  virtual void autoDetectAudioModeSLOT() = 0;
  //  void writeVideoClipFrameSLOT();
  virtual void unShowOSDSLOT();
  virtual void setMuteStateAfterFChangeSLOT() = 0;
  virtual void mouseCursorOffTimerSLOT();

  /* ---------------------------------------------------------------------- */
signals:
  void keyPressedSIGNAL(int key);   // returns all keys pressed in winTVScreenWidget
  void autoDetectAudioModeSIGNAL(int mode);
  void requestMixerMute(bool);
  void toggleTVonlySIGNAL();
  void fullScreenRequest();

  /* ---------------------------------------------------------------------- */
protected:
  static v4lxif * v4lx;            // video4linux interface (v4l, v4l2)

  virtual void resizeEvent( QResizeEvent * );
  virtual void moveEvent( QMoveEvent * );
  virtual void paintEvent (QPaintEvent * ) = 0;
  //void closeEvent (QCloseEvent * );
  virtual bool eventFilter( QObject *, QEvent * ) = 0;
  virtual bool x11Event( XEvent * ) = 0;
  virtual void keyPressEvent ( QKeyEvent *);
  //void mouseMoveEvent (QMouseEvent *);
  virtual void mouseReleaseEvent ( QMouseEvent * me);
  virtual void mouseDoubleClickEvent(QMouseEvent *);

  /* ---------------------------------------------------------------------- */
  virtual void capture( int on ) = 0;
  virtual void switch_capture( int on ) = 0;

  virtual int initializeTV(WId parentWid = 0);
  virtual void destroyTV() = 0;

  virtual void configOverlay() = 0;
  virtual void setTVMaxSize() = 0;
  virtual void refreshTV() = 0;
  virtual int setTVClipTab(int num) = 0;
  virtual int getTVClipTab(bool set,
                           int wx=0, int wy=0,
                           unsigned int ww=0, unsigned int wh=0,
                          int xadj=0, int yadj=0) = 0;

/* --------------------- resources --------------------------------- */

  QWidget *parent;

  const char *devname;      // name of the video4linux-device

  // bttv stuff
  int defaultNorm;                // input norm, PAL, NTSC, SECAM
  int defaultInput;               // input channel, TUNER, COMP1, COMP2, SVHS
  int defaultFreq;                // frequency, *16
  int viewRefreshDelay;               // 200ms
  int Bright;              // Brightness
  int Color;               // intensity of colors
  int Hue;                 // hue
  int Contrast;            // contrast

  bool snap;            // snapshot

  unsigned int max_width;          // max width of TV picture
  unsigned int max_height;         // max height of TV picture, both set by setTVMaxSize

  int alloc_context;
  QColor qcolors[256];     // colormap, needed when having only 256 colors -> pseudocolor

  bool colorMap8;              // are we using an own colormap ( <= 256 colors)

  QTimer *viewRefreshTimer;
  int newVisibility,lastClipCount;
  bool allowViewRefresh,didViewRefresh,allowViewRefreshVisibility;
  bool didViewMove,doNewConfig,mapWindow;            // the widget was moved
  int oldxadj,oldyadj;
  int nnewclips,noldclips;
  int widget_is_active;           // the widget is aktive

  QCursor orgCursor;       // Cursor

  QTimer *audioDetectTimer,*muteTimer,*mouseHideTimer;
  int audioMode,audioVolumeCan,audioVolume;
  int muteDelay;

  int allowCursorHiding,cursorIsVisible,muteIsOn,captureIsOn,captureInitIsOn;

  int shotnumber;          // number of screen shot, starting at 0

  // popup
  KPopupMenu *tvPopup;

  // video clip stuff
//    QTime videoTime;
//    videoClip *video;
//    int video_aktive, video_capturing;
//    bool video_frame;
//    long elapsed;

  // info tools
  QLabel *chInfoText;
  QProgressBar *chInfoBar;
  int chInfo;
  QTimer *infoTimer;
  int infodelay;
  bool infoIsVisible;

  // title image
  QPixmap title_pm;
  int title_shown;

  bool mrm_on;
  int mrm_x,mrm_y,mrm_x1,mrm_x2,mrm_y1,mrm_y2,mrm_dh,mrm_dw;

  int _screen;
  bool mouseWasIn;
  XColor xcolors[256];
  Colormap xcolormap; // needed when having only 256 colors -> pseudocolor
  XErrorHandler oldXErrorHandler;
};

#endif

