/*
    KWinTV Xv widget

    Copyright (C) 2001 George Staikos (staikos@kde.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "wintv_defs.h"

#include <wintv.h>
#include <assert.h>

#include <stdlib.h>
#include <iostream.h>
#include <fstream.h>
#include <strstream.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <linux/errno.h> // for RedHat

#ifdef HAVE_LIBJPEG
  #include <jpeglib.h>
#endif

#include <wintvscreen_xv.h>


#include <kiconloader.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qtextstream.h>
#include <qdatetime.h>

#include <kdebug.h>

// includes needed for X stuff
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/IntrinsicP.h>
#include <X11/ObjectP.h>
#include <X11/StringDefs.h>
#include <X11/Xatom.h>
#include <X11/Xmd.h>

#if defined(HAVE_LIBXXF86VM)
#include <X11/extensions/xf86vmode.h>
#include <X11/extensions/xf86vmstr.h>
#endif

#include <qt_visual.h>

// debug switches
#ifdef myDEBUG
//#define tvDEBUG
//#define X11DEBUG
#define CLIPDEBUG
#include <xeventtype.h>
char * visibility[] = {"Un","Partially","Fully" , NULL };
#endif

#define HEX(N) hex << setw(N) << setfill(int('0'))
#define DEC(N) dec << setw(N) << setfill(int('0'))

extern int errno;  // for RedHat

#define OSD

WinTVScreen_xv::WinTVScreen_xv( QWidget *_parent, const char *name,
				      int _capture,
				      const char *_bttvdev,
				      int _bpp, int _palette,
				      int _freq, int _input )
  : winTVScreenWidget( _parent, name, _capture, _bttvdev, _bpp, _palette, _freq, _input)
{

  // Find the Xv device and get a handle to it
  // do the init
  _xvh = KXv::connect(winId());
  if (_xvh) {
    KXvDeviceList& devs = _xvh->devices();
    for (_xvdev = devs.first(); _xvdev; _xvdev = devs.next()) {
       if (_xvdev->isVideoSource()) break;
       _xvdev = NULL;
    }
    if (_xvdev) {
       kdDebug() << "Taking Xv video source: " << _xvdev->name() << endl;
       _xvdev->startVideo(this, width(), height());
    }
  }    

    // use my XErrorHandler
    oldXErrorHandler = XSetErrorHandler((XErrorHandler) xerror);

    // init the rest
    initializeTV(parent->winId());

    // we are running
    widget_is_active = 1;

}

//==============================================================================

void WinTVScreen_xv::postInit()
{
  winTVScreenWidget::postInit();
}

// ============================================================================

WinTVScreen_xv::~WinTVScreen_xv()
{
kdDebug() << "WinTVScreen_xv destructor called" << endl;
  if (widget_is_active) {
    destroyTV();
  }

  if (_xvh) delete _xvh;
}

// ============================================================================
// X11 clipping stuff

void WinTVScreen_xv::configOverlay()
{
}

void WinTVScreen_xv::Shellx11EventSLOT( XEvent *event )
{
  if ( captureIsOn ) {
    switch ( event->type ) {
    case MapNotify:
      doNewConfig = true;
      mapWindow = true;
      configOverlay();
     break;
    case UnmapNotify:
      doNewConfig = true;
      mapWindow = false;
      configOverlay();
     break;
    } // switch
  }
}

// X11 events for the TV window
bool WinTVScreen_xv::x11Event( XEvent *event )
{
  if ( isVisible() && (event->xvisibility.window = (Window) winId()) ) {
    if ( allowCursorHiding && event->type == MotionNotify ) {
      if ( !cursorIsVisible ) {
	setCursor(orgCursor);
	cursorIsVisible = true;
      }

      if (mouseHideTimer->isActive()) mouseHideTimer->stop();

      mouseHideTimer->start(MOUSE_DELAY,true);

#ifdef HAVE_LIBXXF86VM
      if ( mrm_on ) {
	XF86VidModeGetViewPort(qt_xdisplay(),
                               XDefaultScreen(qt_xdisplay()),
                               &mrm_x, &mrm_y);
	if ( mrm_x != mrm_x1 || mrm_y != mrm_y1 ) {
           XF86VidModeSetViewPort(qt_xdisplay(),
                                  XDefaultScreen(qt_xdisplay()),
                                  mrm_x1, mrm_y1);
           mrm_x = mrm_x1 + mrm_x2/2;
           mrm_y = mrm_y1 + mrm_y2/2;
           QCursor::setPos( mrm_x , mrm_y );
        }
      }
#endif
      return false;
    }

    if ( captureIsOn ) {
      switch ( event->type ) {
      case Expose:
	if ( !event->xexpose.count ) {
	  if (newVisibility == VisibilityUnobscured) {
	    doNewConfig = true;
	    configOverlay();
	  }
	}
	break;
      case VisibilityNotify:
	newVisibility = event->xvisibility.state;
	switch (event->xvisibility.state) {
	case VisibilityFullyObscured:
          if ( !didViewRefresh ) configOverlay();
	  break;
	case VisibilityPartiallyObscured:
	case VisibilityUnobscured:
	  doNewConfig = true;
	  mapWindow = true;
	  if ( didViewRefresh ) {
	    didViewRefresh = false;
	  } else {
	    configOverlay();
	  }
	  break;
	}

      } // switch (event->type)
    }
  }
  return (false);
}


// X11 events for the root window (desktop)
bool WinTVScreen_xv::eventFilter( QObject *, QEvent *e )
{

  X11RWEvent *re = (X11RWEvent*)e;

  if ( captureIsOn && isVisible() &&
       (e->type() == XRWEvent) &&
       (re->XEventPtr()->xvisibility.window != (Window) winId()) ) {

    switch (re->XEventTyp()) {
    case MapNotify:
    case UnmapNotify:
    case ConfigureNotify:

      if ( !didViewRefresh )
	configOverlay();

     break;
    } // switch

    return true;                        // eat event
  }

  return false;                         // standard event processing
}

// ============================================================================

void WinTVScreen_xv::paintEvent( QPaintEvent * event)
{
  if ( !captureIsOn ) {
    if ( !title_shown ) {
      QPainter pa(this);
      pa.setClipRect(event->rect());
      pa.drawPixmap((width()/2) - (title_pm.width()/2),
		    (height()/2) - (title_pm.height()/2),
		    title_pm);
    } else {
      kdDebug() << "Alert:  Please report this to the developer. 4939242" << endl;
    }
  }
}

// ============================================================================

void WinTVScreen_xv::switch_capture(int on)
{
  if (on && captureIsOn) {
    if ( isVisible() ) {
      _xvdev->startVideo(this, width(), height());
      captureIsOn = true;
    }
  } else {
    _xvdev->stopVideo();
  }
}

// ============================================================================

void WinTVScreen_xv::capture(int on)
{
  if (isVisible()) {
    if (on) {
      captureIsOn = true;
      newTVGeometry( false );
    } else {
      captureIsOn = false;
    }
    if (captureIsOn)
      _xvdev->startVideo(this, width(), height());
    else _xvdev->stopVideo();
  }
}

// ============================================================================

int WinTVScreen_xv::initializeTV(WId parentWid)
{
  return winTVScreenWidget::initializeTV(parentWid);
}

// ============================================================================

void WinTVScreen_xv::setTVMaxSize()
{
}

// ===========================================================================

void WinTVScreen_xv::destroyTV()
{
}

// ============================================================================

void WinTVScreen_xv::newTVGeometry( bool doViewRefresh )
{

}

// ============================================================================

void WinTVScreen_xv::autoDetectAudioModeSLOT()
{
}

// ============================================================================

void WinTVScreen_xv::refreshTVActionSLOT()
{
}

// ============================================================================

void  WinTVScreen_xv::refreshTV()
{
}

// ============================================================================

int WinTVScreen_xv::getTVClipTab( bool set,
                                   int wx, int wy,
                                   unsigned int ww, unsigned int wh,
                                   int xadj, int yadj )
{
}

// ============================================================================

int WinTVScreen_xv::setTVClipTab( int num )
{
}

// ============================================================================
// -------------------------------- SLOTS -------------------------------------
// ============================================================================

void WinTVScreen_xv::snapShotSLOT(const char *prefix, int fixed, int width,
                                  int height, int format, int quality)
{
}

// ============================================================================

void WinTVScreen_xv::changeFrequencySLOT(int value)
{
  if ( defaultFreq != value ) {
    defaultFreq = value;
    _xvdev->setAttribute( "XV_FREQ", value );
  }
}

// ============================================================================

void WinTVScreen_xv::changeBrightnessSLOT(int bri)
{
  if (Bright != bri) {
      Bright = bri;
      _xvdev->setAttribute( "XV_BRIGHTNESS", bri );
  }
}

// ============================================================================

void WinTVScreen_xv::changeContrastSLOT(int contr)
{
  if (Contrast != contr) {
      Contrast = contr;
      _xvdev->setAttribute( "XV_CONTRAST", contr );
  }
}

// ============================================================================

void WinTVScreen_xv::changeColorSLOT(int color)
{
  if (Color != color) {
      Color = color;
      _xvdev->setAttribute( "XV_COLORKEY", color );
  }
}

// ============================================================================

void WinTVScreen_xv::changeHueSLOT(int hue)
{
  if (Hue != hue) {
      Hue = hue;
      _xvdev->setAttribute( "XV_HUE", hue );
  }
}

// ============================================================================

void WinTVScreen_xv::changeSourceSLOT(int s)
{
}

// ============================================================================

void WinTVScreen_xv::changeNormSLOT(int n)
{
  // change input norm ( PAL, SECAM, NTSC )
}

// ============================================================================

void WinTVScreen_xv::changeChannelValuesSLOT(channel *c, int id)
{
  if (!c) return;

  switch_capture(0);

#ifdef XV_IS_WORKING
  if ( !muteIsOn && v4lx->capChannelTuner(c->input) ) {
    // not muted -> mute for some milisecs...
    emit requestMixerMute(true);
     v4lx->setAudioMute(true);
    qApp->processEvents( 250 ); // give mixer time to mute soundcard
    usleep(SCAN_DELAY);
    if ( muteTimer->isActive() ) muteTimer->stop();
    muteTimer->start(muteDelay);
  }
#endif

  changeSourceSLOT(c->input);
  changeNormSLOT(c->norm);
/*
  if ( v4lx->capChannelTuner(c->input) ) {
    changeFrequencySLOT(c->frequency);
  }
*/

  changeBrightnessSLOT(c->brightness);
  changeColorSLOT(c->color);
  changeHueSLOT(c->hue);
  changeContrastSLOT(c->contrast);

  switch_capture(1);

  c->id = id;

  if ( chInfo >= OSD_LEVEL_2 ) showOSDSLOT(c);
}

// ============================================================================

void WinTVScreen_xv::setMuteStateAfterFChangeSLOT()
{
  muteTimer->stop();
  if ( !muteIsOn ) { // not muted -> unmute again
    v4lx->setAudioMute(false);
    emit requestMixerMute(false);
  }
}

// ============================================================================

void WinTVScreen_xv::toggleAudioMuteSLOT()
{
  muteIsOn = !muteIsOn;

  //v4lx->setAudioMute(muteIsOn);

  if (muteIsOn) {
    if ( chInfo > OSD_LEVEL_1 ) showOSDSLOT("sound muted");
  } else {
    if ( chInfo > OSD_LEVEL_1 ) showOSDSLOT("sound unmuted");
  }
}

// ============================================================================

void WinTVScreen_xv::changeAudioVolumeSLOT(int vl, int vr)
{
// unused?
}

// ============================================================================

void WinTVScreen_xv::changeAudioModeSLOT(int mode)
{
}


// ============================================================================

void WinTVScreen_xv::showOSDSLOT(channel *ch)
{
#ifdef OSD
  int wx,wy;
  unsigned int wwidth,wheight;

  if ( !captureIsOn || !isVisible() || (chInfo == OSD_LEVEL_OFF) ) return;

  if (infoTimer->isActive()) infoTimer->stop();
  infoTimer->start(infodelay,true);

  //v4lx->getCapAClip( &wx, &wy, &wwidth, &wheight );

  chInfoText->setGeometry(wx+(wwidth/10),wy+wheight-(wheight/10)-10,
			  (wwidth/10 * 8)+7 & ~3,(wheight/10)+7 & ~3);

  QString tout;
  tout.setNum(ch->id + 1);
  tout.insert(0," ");
  tout += " -- ";
  tout += ch->name;
  tout = tout.leftJustify(25,' ');
  tout += QTime::currentTime().toString();
  chInfoText->setText(tout);

  infoIsVisible = true;

  switch_capture(0);
  chInfoText->show();
  switch_capture(1);
#endif
}

// ============================================================================

void WinTVScreen_xv::showOSDSLOT(const char * text)
{
#ifdef OSD
  int wx,wy;
  unsigned int wwidth,wheight;

  if ( !captureIsOn || !isVisible() || (chInfo == OSD_LEVEL_OFF) ) return;

  if (infoTimer->isActive()) infoTimer->stop();

  //v4lx->getCapAClip( &wx, &wy, &wwidth, &wheight );

  chInfoText->setGeometry(wx+(wwidth/10),wy+wheight-(wheight/10)-10,
			  (wwidth/10 * 8)+7 & ~3,(wheight/10)+7 & ~3);

  QString tout(" ");
  tout += text;
  chInfoText->setText(tout);

  infoIsVisible = true;
  switch_capture(0);
  chInfoText->show();
  switch_capture(1);

  infoTimer->start(infodelay,true);
#endif
}


void WinTVScreen_xv::rawSetFreq(unsigned long freq) {
    _xvdev->setAttribute( "XV_FREQ", freq );
}


void WinTVScreen_xv::rawCapTunerRange(unsigned long *fmin, unsigned long *fmax) {
    
}


void WinTVScreen_xv::rawSetChannel(int source) {

}


void WinTVScreen_xv::rawSetChannelNorm(int norm) {

}

unsigned long WinTVScreen_xv::rawGetTunerSignal() {

}



#include "wintvscreen_xv.moc"




