/* 
    KWinTV Xv widget

    Copyright (C) 2001 George Staikos <staikos@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#ifndef WINTVSCREEN_xv_H
#define WINTVSCREEN_xv_H

#include "wintvscreen.h"
#include "kxv.h"

#define STUB(x) { printf("STUB %s called!\n", x); }


class WinTVScreen_xv : public winTVScreenWidget
{
  
  Q_OBJECT

public:
  WinTVScreen_xv( QWidget *_parent=0, const char *name=0,
		     int _capture=0, 
		     const char *_bttvdev=V4L_DEVICE,
		     int _bpp = 0, int _palette = 0,
		     int _freq=DFREQ, int _input=DNORM );
  ~WinTVScreen_xv();
 
  bool haveDevice() const { return (_xvdev != NULL); }
   int win_width() STUB("win_width")
   int win_height() STUB("win_height")

   int getMute() { int x; if (_xvdev->getAttribute("XV_MUTE", &x)) return x; return 0; }
   int getVolume() STUB("getVolume")

   void setPalette( int p ) STUB("setPalette")
   void newTVGeometry( bool doViewRefresh );

   void rawSetFreq(unsigned long freq);
   void rawCapTunerRange(unsigned long *fmin, unsigned long *fmax);
   void rawSetChannel(int source);
   void rawSetChannelNorm(int norm);
   unsigned long rawGetTunerSignal();


  /* -------------------------------------------------------------------- */
public slots:
  // use this slots to change anything 
  void showOSDSLOT(channel *ch);
  void showOSDSLOT(const char *text );

  // tuning et al
  void changeFrequencySLOT(int value);  
  void changeBrightnessSLOT(int bri);      
  void changeContrastSLOT(int contr);
  void changeColorSLOT(int color);
  void changeHueSLOT(int hue);
  void changeSourceSLOT(int s);
  void changeNormSLOT(int n);

  // set all 
  void changeChannelValuesSLOT(channel *c, int id);

   // sound et al
  void toggleAudioMuteSLOT();
  void changeAudioVolumeSLOT(int vl, int vr);
  void changeAudioModeSLOT(int mode);

  // one snap shot
  void snapShotSLOT(const char *prefix, int fixed, int width, int height, int format, int quality);

  // create video clip; format: VIDEOCLIP_AVI, VIDEOCLIP_PPM, VIDEOCLIP_RAW
  //void startVideoClipSLOT(struct MOVIE_PARAMS *par); 
  //void endVideoClipSLOT();

  void postInit();

  void Shellx11EventSLOT(XEvent *);

  /* ----- stop reading --- everything below is private ... -------------- */
private slots:
  void refreshTVActionSLOT();
  void autoDetectAudioModeSLOT();
  //  void writeVideoClipFrameSLOT();
  void setMuteStateAfterFChangeSLOT();

  /* ----------------------------------------------------------------- */
signals:
  void keyPressedSIGNAL(int key);
  void autoDetectAudioModeSIGNAL(int mode);
  void requestMixerMute(bool);
  void toggleTVonlySIGNAL();
  void fullScreenRequest();

  /* ------------------------------------------------------------------ */
protected:
  void paintEvent (QPaintEvent * );
  bool eventFilter( QObject *, QEvent * );
  bool x11Event( XEvent * );

  /* ----------------------------------------------------------------- */
private:
  void capture( int on ); // 1=on 0=off
  void switch_capture( int on ); // switch capture on/off 

  int initializeTV( WId parentWid = 0 );
  void destroyTV();

  void configOverlay();
  void setTVMaxSize();
  void refreshTV();
  int setTVClipTab( int num );
  int getTVClipTab( bool set, 
		    int wx=0, int wy=0, 
		    unsigned int ww=0, unsigned int wh=0, 
		    int xadj=0, int yadj=0);

  KXv* _xvh;
  KXvDevice* _xvdev;

/* ---------------------------- resources --------------------------------- */

};

#endif





