/* 
    Copyright (C) 2001 George Staikos (staikos@kde.org)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#include "xhelpers.h"

#ifdef HAVE_CONFIG_H
  #include <config.h>
#endif

#include <qobject.h>
#include <kdebug.h>

extern "C" {
#include <X11/Xlib.h>
#ifdef HAVE_LIBXINERAMA
#include <X11/extensions/Xinerama.h>
#endif
};



void XineramaGetScreenGeometry(int screen, int *x, int *y, int *w, int *h) {
#ifdef HAVE_LIBXINERAMA
  int dummy_a, dummy_b;
  if (XineramaQueryExtension(qt_xdisplay(), &dummy_a, &dummy_b)) {
    if (XineramaIsActive(qt_xdisplay())) {
      int screens;
      XineramaScreenInfo *si;
      si = XineramaQueryScreens(qt_xdisplay(), &screens);
      if (screens > 1) {
        for (int i = 0; i < screens; i++) {
          if (si[i].screen_number == screen) {
            if (x) *x = si[i].x_org;
            if (y) *y = si[i].y_org;
            if (w) *w = si[i].width;
            if (h) *h = si[i].height;
//kdDebug() << "Screen geometry found for xinerama screen " << screen << endl;
            return;
          }
        }
      }
      if (si) {
        XFree(si);
      }
    }
  }
#endif

  if (x) *x = 0;
  if (y) *y = 0;
  if (w) *w = DisplayWidth(qt_xdisplay(), 0);
  if (h) *h = DisplayHeight(qt_xdisplay(), 0);

//kdDebug() << "Screen geometry found for non-xinerama screen" << endl;
  return;
}


int XineramaGetScreenForPoint(int x, int y) {
#ifdef HAVE_LIBXINERAMA
  int dummy_a, dummy_b;
  if (XineramaQueryExtension(qt_xdisplay(), &dummy_a, &dummy_b)) {
    if (XineramaIsActive(qt_xdisplay())) {
      int screens;
      XineramaScreenInfo *si;
      si = XineramaQueryScreens(qt_xdisplay(), &screens);
      if (screens > 1) {
        for (int i = 0; i < screens; i++) {
          if (x >= si[i].x_org && x < si[i].x_org+si[i].width &&
              y >= si[i].y_org && y < si[i].y_org+si[i].height) {
//kdDebug() << "Screen for " << x << " " << y << " = " << si[i].screen_number << " found" << endl;
            return si[i].screen_number;
          }
        }
      }
      if (si) {
        XFree(si);
      }
    }
  }
#endif

  return XDefaultScreen(qt_xdisplay());
}



int XineramaLeftOf(int screen) {
int rc = -1;
#ifdef HAVE_LIBXINERAMA
  int dummy_a, dummy_b;
  if (XineramaQueryExtension(qt_xdisplay(), &dummy_a, &dummy_b)) {
    if (XineramaIsActive(qt_xdisplay())) {
      int screens;
      XineramaScreenInfo *si;
      si = XineramaQueryScreens(qt_xdisplay(), &screens);
      if (screens > 1) {
        int find = -1;
        for (int i = 0; i < screens; i++) {
          if (si[i].screen_number == screen) {
            find = i;
            break;
          } 
        }
        if (find >= 0)
          for (int i = 0; i < screens; i++) {
            if (si[i].x_org+si[i].width == si[find].x_org &&
                si[i].y_org == si[find].y_org) {
              rc = si[i].screen_number;
              break;
            }
          }
      }
      if (si) {
        XFree(si);
      }
    }
  }

#endif
return rc;
}


int XineramaRightOf(int screen) {
int rc = -1;
#ifdef HAVE_LIBXINERAMA
  int dummy_a, dummy_b;
  if (XineramaQueryExtension(qt_xdisplay(), &dummy_a, &dummy_b)) {
    if (XineramaIsActive(qt_xdisplay())) {
      int screens;
      XineramaScreenInfo *si;
      si = XineramaQueryScreens(qt_xdisplay(), &screens);
      if (screens > 1) {
        int find = -1;
        for (int i = 0; i < screens; i++) {
          if (si[i].screen_number == screen) {
            find = i;
            break;
          } 
        }
        if (find > 0)
          for (int i = 0; i < screens; i++) {
            if (si[find].x_org+si[find].width == si[i].x_org &&
                si[i].y_org == si[find].y_org) {
              rc = si[i].screen_number;
              break;
            }
          }
      }
      if (si) {
        XFree(si);
      }
    }
  }

#endif
return rc;
}


int XineramaAbove(int screen) {
int rc = -1;
#ifdef HAVE_LIBXINERAMA
  int dummy_a, dummy_b;
  if (XineramaQueryExtension(qt_xdisplay(), &dummy_a, &dummy_b)) {
    if (XineramaIsActive(qt_xdisplay())) {
      int screens;
      XineramaScreenInfo *si;
      si = XineramaQueryScreens(qt_xdisplay(), &screens);
      if (screens > 1) {
        int find = -1;
        for (int i = 0; i < screens; i++) {
          if (si[i].screen_number == screen) {
            find = i;
            break;
          } 
        }
        if (find > 0)
          for (int i = 0; i < screens; i++) {
            if (si[i].y_org+si[i].height == si[find].y_org &&
                si[i].x_org == si[find].x_org) {
              rc = si[i].screen_number;
              break;
            }
          }
      }
      if (si) {
        XFree(si);
      }
    }
  }

#endif
return rc;
}


int XineramaBelow(int screen) {
int rc = -1;
#ifdef HAVE_LIBXINERAMA
  int dummy_a, dummy_b;
  if (XineramaQueryExtension(qt_xdisplay(), &dummy_a, &dummy_b)) {
    if (XineramaIsActive(qt_xdisplay())) {
      int screens;
      XineramaScreenInfo *si;
      si = XineramaQueryScreens(qt_xdisplay(), &screens);
      if (screens > 1) {
        int find = -1;
        for (int i = 0; i < screens; i++) {
          if (si[i].screen_number == screen) {
            find = i;
            break;
          } 
        }
        if (find > 0)
          for (int i = 0; i < screens; i++) {
            if (si[find].y_org+si[find].height == si[i].y_org &&
                si[i].x_org == si[find].x_org) {
              rc = si[i].screen_number;
              break;
            }
          }
      }
      if (si) {
        XFree(si);
      }
    }
  }

#endif
return rc;
}


