/*
    KWinTV Screensaver

    Copyright (C) 2001 George Staikos (staikos@kde.org)
    based on blankscrn: Copyright (c)  Martin R. Jones 1996

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <qlabel.h>
#include <qlayout.h>
#include <kapp.h>
#include <klocale.h>
#include <kconfig.h>
#include <kcolordlg.h>
#include <kbuttonbox.h>
#include <kcolorbutton.h>
#include <kglobal.h>
#include "wintvsaver.h"
#include "wintvsaver.moc"


// libkscreensaver interface
extern "C"
{
#include <X11/Xlib.h>
#include <X11/extensions/Xinerama.h>
    const char *kss_applicationName = "kwintvsaver.kss";
    const char *kss_description = I18N_NOOP( "KWinTVSaver" );
    const char *kss_version = "1.0.0";

    KScreenSaver *kss_create( WId id )
    {
        return new KWinTVSaver( id );
    }

    QDialog *kss_setup()
    {
        return new KWinTVSetup();
    }
}

//-----------------------------------------------------------------------------
// dialog to setup screen saver parameters
//
KWinTVSetup::KWinTVSetup( QWidget *parent, const char *name )
	: QDialog( parent, name, TRUE )
{
	readSettings();

	QLabel *label;
	QPushButton *button;

	setCaption( i18n("Setup KWinTV Screen Saver") );

	QVBoxLayout *tl = new QVBoxLayout(this, 10);
	QHBoxLayout *tl1 = new QHBoxLayout;
	tl->addLayout(tl1);

	QVBoxLayout *tl11 = new QVBoxLayout(5);
	tl1->addLayout(tl11);

	label = new QLabel( i18n("No-TV Color:"), this );
	tl11->addWidget(label);

	colorPush = new KColorButton( color, this );
	colorPush->setMinimumWidth(80);
	connect( colorPush, SIGNAL( changed(const QColor &) ),
		SLOT( slotColor(const QColor &) ) );
	tl11->addWidget(colorPush);
	tl11->addStretch(1);

	preview = new QWidget( this );
	preview->setFixedSize( 220, 170 );
	preview->setBackgroundColor( black );
	preview->show();    // otherwise saver does not get correct size
//	saver = new KWinTVSaver( preview->winId() );
	tl1->addWidget(preview);

	KButtonBox *bbox = new KButtonBox(this);	
	bbox->addStretch(1);

	button = bbox->addButton( i18n("OK"));	
	connect( button, SIGNAL( clicked() ), SLOT( slotOkPressed() ) );

	button = bbox->addButton(i18n("Cancel"));
	connect( button, SIGNAL( clicked() ), SLOT( reject() ) );
	bbox->layout();
	tl->addWidget(bbox);

	tl->freeze();
}

// read settings from config file
void KWinTVSetup::readSettings()
{
	KConfig *config = KGlobal::config();
	config->setGroup( "Settings" );

	color = config->readColorEntry( "Color", &black );
}

void KWinTVSetup::slotColor( const QColor &col )
{
    color = col;
    //saver->setColor( color );
}

// Ok pressed - save settings and exit
void KWinTVSetup::slotOkPressed()
{
	KConfig *config = KGlobal::config();
	config->setGroup( "Settings" );

	config->writeEntry( "Color", color );

	config->sync();

	accept();
}

//-----------------------------------------------------------------------------


KWinTVSaver::KWinTVSaver( WId id ) : KScreenSaver( id )
{
bool dontDo = false;

#ifdef HAVE_LIBXINERAMA
	if (XineramaIsActive(qt_xdisplay())) {
		dcc = NULL;
		dontDo = true;
	}
#endif

	if (!dontDo) {
	dcc = new DCOPClient;
	dcc->attach();
	if (!dcc->isApplicationRegistered("kwintv")) {
		delete dcc;
		dcc = NULL;
	}
	}

	readSettings();

	blank();
}

KWinTVSaver::~KWinTVSaver()
{
	if (dcc) {
		QByteArray data, retval;
		QCString rettype;
		dcc->call("kwintv", 
                          "KWinTVIface", 
                          "disableFullScreen()", 
                          data, rettype, retval);
		delete dcc;
	}
}

// set the color
void KWinTVSaver::setColor( const QColor &col )
{
	color = col;
	blank();
}

// read configuration settings from config file
void KWinTVSaver::readSettings()
{
	KConfig *config = KGlobal::config();
	config->setGroup( "Settings" );

	color = config->readColorEntry( "Color", &black );
}

void KWinTVSaver::blank()
{
	if (dcc) {
		QByteArray data, retval;
		QCString rettype;
		dcc->call("kwintv", 
                          "KWinTVIface", 
                          "disableFullScreen()", 
                          data, rettype, retval);
		dcc->call("kwintv", 
                          "KWinTVIface", 
                          "fullScreen()", 
                          data, rettype, retval);
	} else {
		setBackgroundColor( color );
		erase();
	}
}

